@@ -8,26 +8,25 @@ import runpy
 import sys
 import warnings
 from ast import literal_eval
+from itertools import chain, repeat
 from pathlib import Path
 from textwrap import wrap
 from typing import Any, Dict, List
 
+from .utils.translations import trans
+
 
 class InfoAction(argparse.Action):
     def __call__(self, *args, **kwargs):
         # prevent unrelated INFO logs when doing "napari --info"
+        from npe2 import cli
+
         from napari.utils import sys_info
 
         logging.basicConfig(level=logging.WARNING)
         print(sys_info())
-        from .plugins import plugin_manager
-
-        plugin_manager.discover_widgets()
-        if errors := plugin_manager.get_errors():
-            names = {e.plugin_name for e in errors}
-            print("\n!!  Errors were detected in the following plugins:")
-            print("(Run 'napari --plugin-info -v' for more details)")
-            print("\n".join(f"  - {n}" for n in names))
+        print("Plugins:")
+        cli.list(fields="", sort="0", format="compact")
         sys.exit()
 
 
@@ -35,29 +34,13 @@ class PluginInfoAction(argparse.Action):
     def __call__(self, *args, **kwargs):
         # prevent unrelated INFO logs when doing "napari --info"
         logging.basicConfig(level=logging.WARNING)
-        from .plugins import plugin_manager
+        from npe2 import cli
 
-        plugin_manager.discover_widgets()
-        print(plugin_manager)
-
-        if errors := plugin_manager.get_errors():
-            print("!!  Some errors occurred:")
-            verbose = '-v' in sys.argv or '--verbose' in sys.argv
-            if not verbose:
-                print("   (use '-v') to show full tracebacks")
-            print("-" * 38)
-
-            for err in errors:
-                print(err.plugin_name)
-                print(f"  error: {err!r}")
-                print(f"  cause: {err.__cause__!r}")
-                if verbose:
-                    print("  traceback:")
-                    import traceback
-                    from textwrap import indent
-
-                    tb = traceback.format_tb(err.__cause__.__traceback__)
-                    print(indent("".join(tb), '   '))
+        cli.list(
+            fields="name,version,npe2,contributions",
+            sort="name",
+            format="table",
+        )
         sys.exit()
 
 
@@ -154,11 +137,14 @@ def parse_sys_argv():
         '--with',
         dest='with_',
         nargs='+',
+        action='append',
+        default=[],
         metavar=('PLUGIN_NAME', 'WIDGET_NAME'),
         help=(
             "open napari with dock widget from specified plugin name."
             "(If plugin provides multiple dock widgets, widget name must also "
-            "be provided)"
+            "be provided). Use __all__ to open all dock widgets of a "
+            "specified plugin. Multiple widgets are opened in tabs."
         ),
     )
     parser.add_argument(
@@ -184,10 +170,14 @@ def parse_sys_argv():
         nargs=0,
         help='show citation information and exit',
     )
+    # Allow multiple --stack options to be provided.
+    # Each stack option will result in its own stack
     parser.add_argument(
         '--stack',
-        action='store_true',
-        help='concatenate multiple input files into a single stack.',
+        action='append',
+        nargs='*',
+        default=[],
+        help='concatenate multiple input files into a single stack. Can be provided multiple times for multiple stacks.',
     )
     parser.add_argument(
         '--plugin',
@@ -280,21 +270,48 @@ def _run():
 
     else:
         if args.with_:
-            from .plugins import _npe2, plugin_manager
+            from .plugins import _initialize_plugins, _npe2, plugin_manager
 
             # if a plugin widget has been requested, this will fail immediately
             # if the requested plugin/widget is not available.
+            _initialize_plugins()
             plugin_manager.discover_widgets()
-            pname, *wnames = args.with_
-            if wnames:
-                for wname in wnames:
+
+            plugin_manager_plugins = []
+            npe2_plugins = []
+            for plugin in args.with_:
+                pname, *wnames = plugin
+                for _name, (_pname, _wnames) in _npe2.widget_iterator():
+                    if _name == 'dock' and pname == _pname:
+                        npe2_plugins.append(plugin)
+                        if '__all__' in wnames:
+                            wnames = _wnames
+                        break
+
+                for _name, (_pname, _wnames) in plugin_manager.iter_widgets():
+                    if _name == 'dock' and pname == _pname:
+                        plugin_manager_plugins.append(plugin)
+                        if '__all__' in wnames:
+                            # Plugin_manager iter_widgets return wnames as dict keys
+                            wnames = list(_wnames.keys())
+                        print(
+                            trans._(
+                                'Non-npe2 plugin {pname} detected. Disable tabify for this plugin.',
+                                deferred=True,
+                                pname=pname,
+                            )
+                        )
+                        break
+
+                if wnames:
+                    for wname in wnames:
+                        _npe2.get_widget_contribution(
+                            pname, wname
+                        ) or plugin_manager.get_widget(pname, wname)
+                else:
                     _npe2.get_widget_contribution(
-                        pname, wname
-                    ) or plugin_manager.get_widget(pname, wname)
-            else:
-                _npe2.get_widget_contribution(
-                    pname
-                ) or plugin_manager.get_widget(pname)
+                        pname
+                    ) or plugin_manager.get_widget(pname)
 
         from napari._qt.widgets.qt_splash_screen import NapariSplashScreen
 
@@ -308,6 +325,19 @@ def _run():
         # it will collect it and hang napari at start time.
         # in a way that is machine, os, time (and likely weather dependant).
         viewer = Viewer()
+
+        # For backwards compatibility
+        # If the --stack option is provided without additional arguments
+        # just set stack to True similar to the previous store_true action
+        if args.stack and len(args.stack) == 1 and len(args.stack[0]) == 0:
+            warnings.warn(
+                trans._(
+                    "The usage of the --stack option as a boolean is deprecated. Please use '--stack file1 file2 .. fileN' instead. It is now also possible to specify multiple stacks of files to stack '--stack file1 file2 --stack file3 file4 file5 --stack ..'. This warning will become an error in version 0.5.0.",
+                ),
+                DeprecationWarning,
+                stacklevel=3,
+            )
+            args.stack = True
         viewer._window._qt_viewer._qt_open(
             args.paths,
             stack=args.stack,
@@ -317,12 +347,37 @@ def _run():
         )
 
         if args.with_:
-            pname, *wnames = args.with_
-            if wnames:
-                for wname in wnames:
-                    viewer.window.add_plugin_dock_widget(pname, wname)
-            else:
-                viewer.window.add_plugin_dock_widget(pname)
+            # Non-npe2 plugins disappear on tabify or if tabified npe2 plugins are loaded after them.
+            # Therefore, read npe2 plugins first and do not tabify for non-npe2 plugins.
+            for plugin, tabify in chain(
+                zip(npe2_plugins, repeat(True)),
+                zip(plugin_manager_plugins, repeat(False)),
+            ):
+                pname, *wnames = plugin
+                if '__all__' in wnames:
+                    for name, (_pname, _wnames) in chain(
+                        _npe2.widget_iterator(), plugin_manager.iter_widgets()
+                    ):
+                        if name == 'dock' and pname == _pname:
+                            if isinstance(_wnames, dict):
+                                # Plugin_manager iter_widgets return wnames as dict keys
+                                wnames = list(_wnames.keys())
+                            else:
+                                wnames = _wnames
+                            break
+
+                if wnames:
+                    first_dock_widget = viewer.window.add_plugin_dock_widget(
+                        pname, wnames[0], tabify=tabify
+                    )[0]
+                    for wname in wnames[1:]:
+                        viewer.window.add_plugin_dock_widget(
+                            pname, wname, tabify=tabify
+                        )
+                    first_dock_widget.show()
+                    first_dock_widget.raise_()
+                else:
+                    viewer.window.add_plugin_dock_widget(pname, tabify=tabify)
 
         # only necessary in bundled app, but see #3596
         from napari.utils.misc import (
@@ -330,7 +385,7 @@ def _run():
             running_as_bundled_app,
         )
 
-        if running_as_bundled_app:
+        if running_as_bundled_app():
             install_certifi_opener()
         run(gui_exceptions=True)
 
@@ -366,66 +421,74 @@ def _run_plugin_module(mod, plugin_name):
     run()
 
 
-def _run_pythonw(python_path):
-    """Execute this script again through pythonw.
-
-    This can be used to ensure we're using a framework
-    build of Python on macOS, which fixes frozen menubar issues.
-
-    Parameters
-    ----------
-    python_path : pathlib.Path
-        Path to python framework build.
+def _maybe_rerun_with_macos_fixes():
     """
-    import subprocess
+    Apply some fixes needed in macOS, which might involve
+    running this script again using a different sys.executable.
 
-    cwd = Path.cwd()
-    cmd = [python_path, '-m', 'napari']
-    env = os.environ.copy()
+    1) Quick fix for Big Sur Python 3.9 and Qt 5.
+       No relaunch needed.
+    2) Using `pythonw` instead of `python`.
+       This can be used to ensure we're using a framework
+       build of Python on macOS, which fixes frozen menubar issues
+       in some macOS versions.
+    3) Make sure the menu bar uses 'napari' as the display name.
+       This requires relaunching the app from a symlink to the
+       desired python executable, conveniently named 'napari'.
+    """
+    if sys.platform != "darwin":
+        return
 
-    # Append command line arguments.
-    if len(sys.argv) > 1:
-        cmd.extend(sys.argv[1:])
+    if "_NAPARI_RERUN_WITH_FIXES" in os.environ:
+        # This function already ran, do not recurse!
+        # We also restore sys.executable to its initial value,
+        # if we used a symlink
+        if exe := os.environ.pop("_NAPARI_SYMLINKED_EXECUTABLE", ""):
+            sys.executable = exe
+        return
 
-    result = subprocess.run(cmd, env=env, cwd=cwd)
-    sys.exit(result.returncode)
-
-
-def main():
-    # Ensure we're always using a "framework build" on the latest
-    # macOS to ensure menubar works without needing to refocus napari.
-    # We try this for macOS later than the Catelina release
-    # See https://github.com/napari/napari/pull/1554 and
-    # https://github.com/napari/napari/issues/380#issuecomment-659656775
-    # and https://github.com/ContinuumIO/anaconda-issues/issues/199
     import platform
+    import subprocess
+    from tempfile import mkdtemp
 
-    _MACOS_AT_LEAST_CATALINA = (
-        sys.platform == "darwin" and int(platform.release().split('.')[0]) > 19
-    )
-    _MACOS_AT_LEAST_BIG_SUR = (
-        sys.platform == "darwin" and int(platform.release().split('.')[0]) > 20
-    )
+    from qtpy import API_NAME
 
+    # In principle, we will relaunch to the same python we were using
+    executable = sys.executable
+    cwd = Path.cwd()
+
+    _MACOS_AT_LEAST_CATALINA = int(platform.release().split('.')[0]) >= 19
+    _MACOS_AT_LEAST_BIG_SUR = int(platform.release().split('.')[0]) >= 20
     _RUNNING_CONDA = "CONDA_PREFIX" in os.environ
     _RUNNING_PYTHONW = "PYTHONEXECUTABLE" in os.environ
 
-    # quick fix for Big Sur py3.9
-    if _MACOS_AT_LEAST_BIG_SUR:
+    # 1) quick fix for Big Sur py3.9 and qt 5
+    # https://github.com/napari/napari/pull/1894
+    if _MACOS_AT_LEAST_BIG_SUR and '6' not in API_NAME:
         os.environ['QT_MAC_WANTS_LAYER'] = '1'
 
+    # Create the env copy now because the following changes
+    # should not persist in the current process in case
+    # we do not run the subprocess!
+    env = os.environ.copy()
+
+    # 2) Ensure we're always using a "framework build" on the latest
+    # macOS to ensure menubar works without needing to refocus napari.
+    # We try this for macOS later than the Catalina release
+    # See https://github.com/napari/napari/pull/1554 and
+    # https://github.com/napari/napari/issues/380#issuecomment-659656775
+    # and https://github.com/ContinuumIO/anaconda-issues/issues/199
     if (
         _MACOS_AT_LEAST_CATALINA
         and not _MACOS_AT_LEAST_BIG_SUR
         and _RUNNING_CONDA
         and not _RUNNING_PYTHONW
     ):
-        python_path = Path(sys.exec_prefix) / 'bin' / 'pythonw'
-
-        if python_path.exists():
-            # Running again with pythonw will exit this script
-            # and use the framework build of python.
-            _run_pythonw(python_path)
+        pythonw_path = Path(sys.exec_prefix) / 'bin' / 'pythonw'
+        if pythonw_path.exists():
+            # Use this one instead of sys.executable to relaunch
+            # the subprocess
+            executable = pythonw_path
         else:
             msg = (
                 'pythonw executable not found.\n'
@@ -437,7 +500,59 @@ def main():
             )
             warnings.warn(msg)
 
+    # 3) Make sure the app name in the menu bar is 'napari', not 'python'
+    tempdir = None
+    _NEEDS_SYMLINK = (
+        # When napari is launched from the conda bundle shortcut
+        # it already has the right 'napari' name in the app title
+        # and __CFBundleIdentifier is set to 'com.napari._(<version>)'
+        "napari" not in os.environ.get("__CFBundleIdentifier", "")
+        # with a sys.executable named napari,
+        # macOS should have picked the right name already
+        or os.path.basename(executable) != "napari"
+    )
+    if _NEEDS_SYMLINK:
+        tempdir = mkdtemp(prefix="symlink-to-fix-macos-menu-name-")
+        # By using a symlink with basename napari
+        # we make macOS take 'napari' as the program name
+        napari_link = os.path.join(tempdir, "napari")
+        os.symlink(executable, napari_link)
+        # Pass original executable to the subprocess so it can restore it later
+        env["_NAPARI_SYMLINKED_EXECUTABLE"] = executable
+        executable = napari_link
+
+    # if at this point 'executable' is different from 'sys.executable', we
+    # need to launch the subprocess to apply the fixes
+    if sys.executable != executable:
+        env["_NAPARI_RERUN_WITH_FIXES"] = "1"
+        if Path(sys.argv[0]).name == "napari":
+            # launched through entry point, we do that again to avoid
+            # issues with working directory getting into sys.path (#5007)
+            cmd = [executable, sys.argv[0]]
+        else:  # we assume it must have been launched via '-m' syntax
+            cmd = [executable, "-m", "napari"]
+
+        # Append original command line arguments.
+        if len(sys.argv) > 1:
+            cmd.extend(sys.argv[1:])
+        try:
+            result = subprocess.run(cmd, env=env, cwd=cwd)
+            sys.exit(result.returncode)
+        finally:
+            if tempdir is not None:
+                import shutil
+
+                shutil.rmtree(tempdir)
+
+
+def main():
+    # There a number of macOS issues we can fix with env vars
+    # and/or relaunching a subprocess
+    _maybe_rerun_with_macos_fixes()
+
     # Prevent https://github.com/napari/napari/issues/3415
+    # This one fix is needed _after_ a potential relaunch,
+    # that's why it's here and not in _maybe_rerun_with_macos_fixes()
     if sys.platform == "darwin":
         import multiprocessing
 
