@@ -1,10 +1,13 @@
+import contextlib
 import os
 import sys
-from importlib.metadata import metadata
+from enum import Enum, auto
+from importlib.metadata import PackageNotFoundError, metadata
 from pathlib import Path
-from typing import Callable, Dict, List, Optional, Sequence, Tuple
+from tempfile import gettempdir
+from typing import Callable, Dict, List, Literal, Optional, Sequence, Tuple
 
-from npe2.manifest.package_metadata import PackageMetadata
+from npe2 import PackageMetadata, PluginManager
 from qtpy.QtCore import (
     QEvent,
     QObject,
@@ -34,7 +37,6 @@ from qtpy.QtWidgets import (
     QWidget,
 )
 from superqt import QElidingLabel
-from typing_extensions import Literal
 
 import napari.resources
 
@@ -55,7 +57,7 @@ from ..qthreading import create_worker
 from ..widgets.qt_message_popup import WarnPopup
 from ..widgets.qt_tooltip import QtToolTipLabel
 
-InstallerTypes = Literal['pip', 'conda', 'mamba']
+InstallerTypes = Literal['pip', 'mamba']
 
 
 # TODO: add error icon and handle pip install errors
@@ -87,26 +89,55 @@ class Installer(QObject):
         installer: InstallerTypes = "pip",
     ):
         process = QProcess()
-        if installer != "pip":
-            process.setProgram(installer)
-        else:
-            process.setProgram(self._sys_executable_or_bundled_python())
-
         process.setProcessChannelMode(QProcess.MergedChannels)
         process.readyReadStandardOutput.connect(
             lambda process=process: self._on_stdout_ready(process)
         )
+        env = QProcessEnvironment.systemEnvironment()
+
+        if installer == "pip":
+            process.setProgram(self._sys_executable_or_bundled_python())
+            # patch process path
+            combined_paths = os.pathsep.join(
+                [
+                    user_site_packages(),
+                    env.systemEnvironment().value("PYTHONPATH"),
+                ]
+            )
+            env.insert("PYTHONPATH", combined_paths)
+        else:
+            process.setProgram(installer)
+
+        if installer == "mamba":
+            from ..._version import version_tuple
+
+            # To avoid napari version changing when installing a plugin, we
+            # add a pin to the current napari version, that way we can
+            # restrict any changes to the actual napari application.
+            # Conda/mamba also pin python by default, so we effectively
+            # constrain python and napari versions from changing, when
+            # installing plugins inside the constructor bundled application.
+            # See: https://docs.conda.io/projects/conda/en/latest/user-guide/tasks/manage-pkgs.html#preventing-packages-from-updating-pinning
+            napari_version = ".".join(str(v) for v in version_tuple[:3])
+            if env.contains("CONDA_PINNED_PACKAGES"):
+                # str delimiter is '&'
+                system_pins = f"&{env.value('CONDA_PINNED_PACKAGES')}"
+            else:
+                system_pins = ""
+            env.insert(
+                "CONDA_PINNED_PACKAGES",
+                f"napari={napari_version}{system_pins}",
+            )
+            if os.name == "nt":
+                # workaround https://github.com/napari/napari/issues/4247, 4484
+                if not env.contains("TEMP"):
+                    temp = gettempdir()
+                    env.insert("TMP", temp)
+                    env.insert("TEMP", temp)
+                if not env.contains("USERPROFILE"):
+                    env.insert("HOME", os.path.expanduser("~"))
+                    env.insert("USERPROFILE", os.path.expanduser("~"))
 
-        # setup process path
-        env = QProcessEnvironment()
-        combined_paths = os.pathsep.join(
-            [user_site_packages(), env.systemEnvironment().value("PYTHONPATH")]
-        )
-        env.insert("PYTHONPATH", combined_paths)
-        # use path of parent process
-        env.insert(
-            "PATH", QProcessEnvironment.systemEnvironment().value("PATH")
-        )
         process.setProcessEnvironment(env)
         self.set_output_widget(self._output_widget)
         process.finished.connect(
@@ -133,10 +164,11 @@ class Installer(QObject):
         if exit_code != 0:
             self._exit_code = 0
 
-        process_to_terminate = []
-        for pkg_list, proc in self._processes.items():
-            if proc == process:
-                process_to_terminate.append(pkg_list)
+        process_to_terminate = [
+            pkg_list
+            for pkg_list, proc in self._processes.items()
+            if proc == process
+        ]
 
         for pkg_list in process_to_terminate:
             process = self._processes.pop(pkg_list)
@@ -172,10 +204,10 @@ class Installer(QObject):
         installer = installer or self._installer_type
         self._queue.insert(
             0,
-            [
+            (
                 tuple(pkg_list),
                 lambda: self._install(pkg_list, installer, channels),
-            ],
+            ),
         )
         self._handle_action()
 
@@ -186,21 +218,8 @@ class Installer(QObject):
         channels: Sequence[str] = ("conda-forge",),
     ):
         installer = installer or self._installer_type
-        process_environment = QProcessEnvironment.systemEnvironment()
+        process = self._create_process(installer)
         if installer != "pip":
-            from ..._version import version_tuple
-
-            # To avoid napari version changing when installing a plugin, we
-            # add a pin to the current napari version, that way we can
-            # restrict any changes to the actual napari application.
-            # Conda/mamba also pin python by default, so we effectively
-            # constrain python and napari versions from changing, when
-            # installing plugins inside the constructor bundled application.
-            # See: https://docs.conda.io/projects/conda/en/latest/user-guide/tasks/manage-pkgs.html#preventing-packages-from-updating-pinning
-            napari_version = ".".join(str(v) for v in version_tuple[:3])
-            process_environment.insert(
-                "CONDA_PINNED_PACKAGES", f"napari={napari_version}"
-            )
             cmd = [
                 'install',
                 '-y',
@@ -223,8 +242,6 @@ class Installer(QObject):
                 user_plugin_dir(),
             ]
 
-        process = self._create_process(installer)
-        process.setProcessEnvironment(process_environment)
         process.setArguments(cmd + list(pkg_list))
         if self._output_widget and self._queue:
             self._output_widget.clear()
@@ -241,10 +258,10 @@ class Installer(QObject):
         installer = installer or self._installer_type
         self._queue.insert(
             0,
-            [
+            (
                 tuple(pkg_list),
                 lambda: self._uninstall(pkg_list, installer, channels),
-            ],
+            ),
         )
         self._handle_action()
 
@@ -275,8 +292,13 @@ class Installer(QObject):
 
         process.start()
 
+        pm2 = PluginManager.instance()
+
         for pkg in pkg_list:
-            plugin_manager.unregister(pkg)
+            if pkg in pm2:
+                pm2.unregister(pkg)
+            else:
+                plugin_manager.unregister(pkg)
 
         return process
 
@@ -290,11 +312,9 @@ class Installer(QObject):
 
             self._processes = {}
         else:
-            try:
+            with contextlib.suppress(KeyError):
                 process = self._processes.pop(tuple(pkg_list))
                 process.terminate()
-            except KeyError:
-                pass
 
     @staticmethod
     def _is_installed_with_conda():
@@ -312,16 +332,12 @@ class Installer(QObject):
         if conda_meta_path.is_dir():
             for file in conda_meta_path.iterdir():
                 fname = file.parts[-1]
-                if fname.startswith(napari_version_string) and fname.endswith(
-                    ".json"
-                ):
+                if (
+                    fname.startswith(napari_version_string)
+                    or fname.startswith(qt_version_string)
+                ) and fname.endswith(".json"):
                     return True
-                elif fname.startswith(qt_version_string) and fname.endswith(
-                    ".json"
-                ):
-                    return True
-            else:
-                return False
+        return False
 
 
 class PluginListItem(QFrame):
@@ -350,8 +366,7 @@ class PluginListItem(QFrame):
 
         self.help_button.setText(trans._("Website"))
         self.help_button.setObjectName("help_button")
-        if npe_version != 1:
-            self._handle_npe2_plugin()
+        self._handle_npe2_plugin(npe_version)
 
         if installed:
             self.enabled_checkbox.show()
@@ -362,18 +377,18 @@ class PluginListItem(QFrame):
             self.action_button.setText(trans._("install"))
             self.action_button.setObjectName("install_button")
 
-    def _handle_npe2_plugin(self):
+    def _handle_npe2_plugin(self, npe_version):
+        if npe_version == 1:
+            return
+        opacity = 0.4 if npe_version == 'shim' else 1
+        lbl = trans._('npe1 (adapted)') if npe_version == 'shim' else 'npe2'
         npe2_icon = QLabel(self)
         icon = QColoredSVGIcon.from_resources('logo_silhouette')
-        npe2_icon.setPixmap(icon.colored(color='#33F0FF').pixmap(20, 20))
-        self.row1.insertWidget(2, QLabel('npe2'))
-        self.row1.insertWidget(2, npe2_icon)
-        self.enabled_checkbox.setEnabled(False)
-        self.enabled_checkbox.setToolTip(
-            trans._(
-                'This is a npe2 plugin and cannot be enabled/disabled at this time.'
-            )
+        npe2_icon.setPixmap(
+            icon.colored(color='#33F0FF', opacity=opacity).pixmap(20, 20)
         )
+        self.row1.insertWidget(2, QLabel(lbl))
+        self.row1.insertWidget(2, npe2_icon)
 
     def _get_dialog(self) -> QDialog:
         p = self.parent()
@@ -443,7 +458,9 @@ class PluginListItem(QFrame):
 
         self.package_name = QLabel(self)
         self.package_name.setAlignment(
-            Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter
+            Qt.AlignmentFlag.AlignRight
+            | Qt.AlignmentFlag.AlignTrailing
+            | Qt.AlignmentFlag.AlignVCenter
         )
         self.row1.addWidget(self.package_name)
 
@@ -473,7 +490,7 @@ class PluginListItem(QFrame):
         self.row2 = QHBoxLayout()
         self.error_indicator = QPushButton()
         self.error_indicator.setObjectName("warning_icon")
-        self.error_indicator.setCursor(Qt.PointingHandCursor)
+        self.error_indicator.setCursor(Qt.CursorShape.PointingHandCursor)
         self.error_indicator.hide()
         self.row2.addWidget(self.error_indicator)
         self.row2.setContentsMargins(-1, 4, 0, -1)
@@ -504,10 +521,15 @@ class PluginListItem(QFrame):
     def _on_enabled_checkbox(self, state: int):
         """Called with `state` when checkbox is clicked."""
         enabled = bool(state)
-        current_distname = self.plugin_name.text()
-        for plugin_name, _, distname in plugin_manager.iter_available():
-            if distname and distname == current_distname:
-                plugin_manager.set_blocked(plugin_name, not enabled)
+        plugin_name = self.plugin_name.text()
+        pm2 = PluginManager.instance()
+        if plugin_name in pm2:
+            pm2.enable(plugin_name) if state else pm2.disable(plugin_name)
+            return
+
+        for npe1_name, _, distname in plugin_manager.iter_available():
+            if distname and (normalized_name(distname) == plugin_name):
+                plugin_manager.set_blocked(npe1_name, not enabled)
 
     def show_warning(self, message: str = ""):
         """Show warning icon and tooltip."""
@@ -545,20 +567,21 @@ class QPluginList(QListWidget):
         enabled=True,
         npe_version=1,
     ):
+        pkg_name = project_info.name
         # don't add duplicates
         if (
-            self.findItems(project_info.name, Qt.MatchFixedString)
+            self.findItems(pkg_name, Qt.MatchFlag.MatchFixedString)
             and not plugin_name
         ):
             return
 
         # including summary here for sake of filtering below.
-        searchable_text = project_info.name + " " + project_info.summary
-        item = QListWidgetItem(searchable_text, parent=self)
+        searchable_text = f"{pkg_name} {project_info.summary}"
+        item = QListWidgetItem(searchable_text, self)
         item.version = project_info.version
         super().addItem(item)
         widg = PluginListItem(
-            package_name=project_info.name,
+            package_name=pkg_name,
             version=project_info.version,
             url=project_info.home_page,
             summary=project_info.summary,
@@ -584,24 +607,21 @@ class QPluginList(QListWidget):
             )
         else:
             widg.help_button.setVisible(False)
-
         widg.action_button.clicked.connect(
-            lambda: self.handle_action(item, project_info.name, action_name)
+            lambda: self.handle_action(item, pkg_name, action_name)
         )
         widg.update_btn.clicked.connect(
-            lambda: self.handle_action(
-                item, project_info.name, "install", update=True
-            )
+            lambda: self.handle_action(item, pkg_name, "install", update=True)
         )
         widg.cancel_btn.clicked.connect(
-            lambda: self.handle_action(item, project_info.name, "cancel")
+            lambda: self.handle_action(item, pkg_name, "cancel")
         )
         item.setSizeHint(widg.sizeHint())
         self.setItemWidget(item, widg)
 
     def handle_action(self, item, pkg_name, action_name, update=False):
         widget = item.widget
-        item.setText("0-" + item.text())
+        item.setText(f"0-{item.text()}")
         method = getattr(self.installer, action_name)
         self._remove_list.append((pkg_name, item))
         self._warn_dialog = None
@@ -623,6 +643,9 @@ class QPluginList(QListWidget):
 
         if action_name == "install":
             if update:
+                if hasattr(item, 'latest_version'):
+                    pkg_name += f"=={item.latest_version}"
+
                 widget.set_busy(trans._("updating..."), update)
                 widget.action_button.setDisabled(True)
             else:
@@ -643,9 +666,14 @@ class QPluginList(QListWidget):
             widget.set_busy(trans._("cancelling..."), update)
             method((pkg_name,))
 
-    @Slot(PackageMetadata)
-    def tag_outdated(self, project_info: PackageMetadata):
-        for item in self.findItems(project_info.name, Qt.MatchStartsWith):
+    @Slot(PackageMetadata, bool)
+    def tag_outdated(self, project_info: PackageMetadata, is_available: bool):
+        if not is_available:
+            return
+
+        for item in self.findItems(
+            project_info.name, Qt.MatchFlag.MatchStartsWith
+        ):
             current = item.version
             latest = project_info.version
             if parse_version(current) >= parse_version(latest):
@@ -655,6 +683,7 @@ class QPluginList(QListWidget):
                 continue
 
             item.outdated = True
+            item.latest_version = latest
             widg = self.itemWidget(item)
             widg.update_btn.setVisible(True)
             widg.update_btn.setText(
@@ -668,7 +697,9 @@ class QPluginList(QListWidget):
         This will disable the item and the install button and add a warning
         icon with a hover tooltip.
         """
-        for item in self.findItems(project_info.name, Qt.MatchStartsWith):
+        for item in self.findItems(
+            project_info.name, Qt.MatchFlag.MatchStartsWith
+        ):
             widget = self.itemWidget(item)
             widget.show_warning(
                 trans._(
@@ -686,7 +717,10 @@ class QPluginList(QListWidget):
         if text:
             # PySide has some issues, so we compare using id
             # See: https://bugreports.qt.io/browse/PYSIDE-74
-            shown = [id(it) for it in self.findItems(text, Qt.MatchContains)]
+            shown = [
+                id(it)
+                for it in self.findItems(text, Qt.MatchFlag.MatchContains)
+            ]
             for i in range(self.count()):
                 item = self.item(i)
                 item.setHidden(id(item) not in shown)
@@ -696,14 +730,19 @@ class QPluginList(QListWidget):
                 item.setHidden(False)
 
 
+class RefreshState(Enum):
+    REFRESHING = auto()
+    OUTDATED = auto()
+    DONE = auto()
+
+
 class QtPluginDialog(QDialog):
     def __init__(self, parent=None):
         super().__init__(parent)
+        self.refresh_state = RefreshState.DONE
         self.already_installed = set()
 
-        installer_type = "pip"
-        if running_as_constructor_app():
-            installer_type = "mamba" if os.name != "nt" else "conda"
+        installer_type = "mamba" if running_as_constructor_app() else "pip"
 
         self.installer = Installer(installer=installer_type)
         self.setup_ui()
@@ -734,20 +773,28 @@ class QtPluginDialog(QDialog):
         event.ignore()
 
     def refresh(self):
+        if self.refresh_state != RefreshState.DONE:
+            self.refresh_state = RefreshState.OUTDATED
+            return
+        self.refresh_state = RefreshState.REFRESHING
         self.installed_list.clear()
         self.available_list.clear()
 
         # fetch installed
-        from ...plugins import _npe2, plugin_manager
+        from npe2 import PluginManager
 
-        plugin_manager.discover()  # since they might not be loaded yet
+        from ...plugins import plugin_manager
 
         self.already_installed = set()
 
         def _add_to_installed(distname, enabled, npe_version=1):
             norm_name = normalized_name(distname or '')
             if distname:
-                meta = metadata(distname)
+                try:
+                    meta = metadata(distname)
+                except PackageNotFoundError:
+                    self.refresh_state = RefreshState.OUTDATED
+                    return  # a race condition has occurred and the package is uninstalled by another thread
                 if len(meta) == 0:
                     # will not add builtins.
                     return
@@ -770,12 +817,18 @@ class QtPluginDialog(QDialog):
                 npe_version=npe_version,
             )
 
-        for manifest in _npe2.iter_manifests():
+        pm2 = PluginManager.instance()
+        discovered = pm2.discover()
+        for manifest in pm2.iter_manifests():
             distname = normalized_name(manifest.name or '')
             if distname in self.already_installed or distname == 'napari':
                 continue
-            _add_to_installed(distname, True, npe_version=2)
+            enabled = not pm2.is_disabled(manifest.name)
+            # if it's an Npe1 adaptor, call it v1
+            npev = 'shim' if manifest.npe1_shim else 2
+            _add_to_installed(distname, enabled, npe_version=npev)
 
+        plugin_manager.discover()  # since they might not be loaded yet
         for (
             plugin_name,
             _mod_name,
@@ -784,7 +837,7 @@ class QtPluginDialog(QDialog):
             # not showing these in the plugin dialog
             if plugin_name in ('napari_plugin_engine',):
                 continue
-            if distname in self.already_installed:
+            if normalized_name(distname or '') in self.already_installed:
                 continue
             _add_to_installed(
                 distname, not plugin_manager.is_blocked(plugin_name)
@@ -815,16 +868,25 @@ class QtPluginDialog(QDialog):
         self.worker.yielded.connect(self._handle_yield)
         self.worker.finished.connect(self.working_indicator.hide)
         self.worker.finished.connect(self._update_count_in_label)
+        self.worker.finished.connect(self._end_refresh)
         self.worker.start()
+        if discovered:
+            message = trans._(
+                'When installing/uninstalling npe2 plugins, you must restart napari for UI changes to take effect.'
+            )
+            self._warn_dialog = WarnPopup(
+                text=message,
+            )
+            self._warn_dialog.exec_()
 
     def setup_ui(self):
         self.resize(1080, 640)
         vlay_1 = QVBoxLayout(self)
         self.h_splitter = QSplitter(self)
         vlay_1.addWidget(self.h_splitter)
-        self.h_splitter.setOrientation(Qt.Horizontal)
+        self.h_splitter.setOrientation(Qt.Orientation.Horizontal)
         self.v_splitter = QSplitter(self.h_splitter)
-        self.v_splitter.setOrientation(Qt.Vertical)
+        self.v_splitter.setOrientation(Qt.Orientation.Vertical)
         self.v_splitter.setMinimumWidth(500)
 
         installed = QWidget(self.v_splitter)
@@ -926,6 +988,12 @@ class QtPluginDialog(QDialog):
             trans._("Available Plugins ({count})", count=count)
         )
 
+    def _end_refresh(self):
+        refresh_state = self.refresh_state
+        self.refresh_state = RefreshState.DONE
+        if refresh_state == RefreshState.OUTDATED:
+            self.refresh()
+
     def eventFilter(self, watched, event):
         if event.type() == QEvent.DragEnter:
             # we need to accept this event explicitly to be able
@@ -962,7 +1030,7 @@ class QtPluginDialog(QDialog):
     def _handle_yield(self, data: Tuple[PackageMetadata, bool]):
         project_info, is_available = data
         if project_info.name in self.already_installed:
-            self.installed_list.tag_outdated(project_info)
+            self.installed_list.tag_outdated(project_info, is_available)
         else:
             self.available_list.addItem(project_info)
             if not is_available:
@@ -970,7 +1038,7 @@ class QtPluginDialog(QDialog):
 
         self.filter()
 
-    def filter(self, text: str = None) -> None:
+    def filter(self, text: Optional[str] = None) -> None:
         """Filter by text or set current text as filter."""
         if text is None:
             text = self.packages_filter.text()
