@@ -1,3 +1,5 @@
+from typing import TYPE_CHECKING
+
 import numpy as np
 from qtpy.QtCore import Qt
 from qtpy.QtGui import QColor, QPainter
@@ -21,13 +23,16 @@ from ...layers.labels._labels_utils import get_dtype
 from ...utils._dtype import get_dtype_limits
 from ...utils.action_manager import action_manager
 from ...utils.events import disconnect_events
-from ...utils.interactions import Shortcut
 from ...utils.translations import trans
 from ..utils import disable_with_opacity
 from ..widgets._slider_compat import QSlider
 from ..widgets.qt_mode_buttons import QtModePushButton, QtModeRadioButton
 from .qt_layer_controls_base import QtLayerControls
 
+if TYPE_CHECKING:
+    import napari.layers
+
+
 INT32_MAX = 2**31 - 1
 
 
@@ -50,8 +55,6 @@ class QtLabelsControls(QtLayerControls):
         Checkbox to control if label layer is contiguous.
     fill_button : qtpy.QtWidgets.QtModeRadioButton
         Button to select FILL mode on Labels layer.
-    grid_layout : qtpy.QtWidgets.QGridLayout
-        Layout of Qt widget controls for the layer.
     layer : napari.layers.Labels
         An instance of a napari Labels layer.
     ndimSpinBox : qtpy.QtWidgets.QSpinBox
@@ -75,6 +78,8 @@ class QtLabelsControls(QtLayerControls):
         FILL.
     """
 
+    layer: 'napari.layers.Labels'
+
     def __init__(self, layer):
         super().__init__(layer)
 
@@ -102,11 +107,11 @@ class QtLabelsControls(QtLayerControls):
         self.selectionSpinBox.setRange(*dtype_lims)
         self.selectionSpinBox.setKeyboardTracking(False)
         self.selectionSpinBox.valueChanged.connect(self.changeSelection)
-        self.selectionSpinBox.setAlignment(Qt.AlignCenter)
+        self.selectionSpinBox.setAlignment(Qt.AlignmentFlag.AlignCenter)
         self._on_selected_label_change()
 
-        sld = QSlider(Qt.Horizontal)
-        sld.setFocusPolicy(Qt.NoFocus)
+        sld = QSlider(Qt.Orientation.Horizontal)
+        sld.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         sld.setMinimum(1)
         sld.setMaximum(40)
         sld.setSingleStep(1)
@@ -127,7 +132,7 @@ class QtLabelsControls(QtLayerControls):
         ndim_sb.setMinimum(2)
         ndim_sb.setMaximum(self.layer.ndim)
         ndim_sb.setSingleStep(1)
-        ndim_sb.setAlignment(Qt.AlignCenter)
+        ndim_sb.setAlignment(Qt.AlignmentFlag.AlignCenter)
         self._on_n_edit_dimensions_change()
 
         self.contourSpinBox = QLargeIntSpinBox()
@@ -135,7 +140,7 @@ class QtLabelsControls(QtLayerControls):
         self.contourSpinBox.setToolTip(trans._('display contours of labels'))
         self.contourSpinBox.valueChanged.connect(self.change_contour)
         self.contourSpinBox.setKeyboardTracking(False)
-        self.contourSpinBox.setAlignment(Qt.AlignCenter)
+        self.contourSpinBox.setAlignment(Qt.AlignmentFlag.AlignCenter)
         self._on_contour_change()
 
         preserve_labels_cb = QCheckBox()
@@ -155,7 +160,7 @@ class QtLabelsControls(QtLayerControls):
 
         # shuffle colormap button
         self.colormapUpdate = QtModePushButton(
-            None,
+            layer,
             'shuffle',
             slot=self.changeColor,
             tooltip=trans._('shuffle colors'),
@@ -189,10 +194,6 @@ class QtLabelsControls(QtLayerControls):
         action_manager.bind_button(
             'napari:activate_fill_mode',
             self.fill_button,
-            extra_tooltip_text=trans._(
-                "Toggle with {shortcut}",
-                shortcut=Shortcut("Control"),
-            ),
         )
 
         self.erase_button = QtModeRadioButton(
@@ -203,10 +204,6 @@ class QtLabelsControls(QtLayerControls):
         action_manager.bind_button(
             'napari:activate_label_erase_mode',
             self.erase_button,
-            extra_tooltip_text=trans._(
-                "Toggle with {shortcut}",
-                shortcut=Shortcut("Alt"),
-            ),
         )
 
         # don't bind with action manager as this would remove "Toggle with {shortcut}"
@@ -234,10 +231,10 @@ class QtLabelsControls(QtLayerControls):
         rendering_options = [i.value for i in LabelsRendering]
         renderComboBox.addItems(rendering_options)
         index = renderComboBox.findText(
-            self.layer.rendering, Qt.MatchFixedString
+            self.layer.rendering, Qt.MatchFlag.MatchFixedString
         )
         renderComboBox.setCurrentIndex(index)
-        renderComboBox.activated[str].connect(self.changeRendering)
+        renderComboBox.currentTextChanged.connect(self.changeRendering)
         self.renderComboBox = renderComboBox
         self.renderLabel = QLabel(trans._('rendering:'))
         self._on_ndisplay_change()
@@ -261,38 +258,22 @@ class QtLabelsControls(QtLayerControls):
         color_layout.addWidget(self.colorBox)
         color_layout.addWidget(self.selectionSpinBox)
 
-        # grid_layout created in QtLayerControls
-        # addWidget(widget, row, column, [row_span, column_span])
-        self.grid_layout.addLayout(button_row, 0, 0, 1, 4)
-        self.grid_layout.addWidget(QLabel(trans._('label:')), 1, 0, 1, 1)
-        self.grid_layout.addLayout(color_layout, 1, 1, 1, 3)
-        self.grid_layout.addWidget(QLabel(trans._('opacity:')), 2, 0, 1, 1)
-        self.grid_layout.addWidget(self.opacitySlider, 2, 1, 1, 3)
-        self.grid_layout.addWidget(QLabel(trans._('brush size:')), 3, 0, 1, 1)
-        self.grid_layout.addWidget(self.brushSizeSlider, 3, 1, 1, 3)
-        self.grid_layout.addWidget(QLabel(trans._('blending:')), 5, 0, 1, 1)
-        self.grid_layout.addWidget(self.blendComboBox, 5, 1, 1, 3)
-        self.grid_layout.addWidget(self.renderLabel, 6, 0, 1, 1)
-        self.grid_layout.addWidget(self.renderComboBox, 6, 1, 1, 3)
-        self.grid_layout.addWidget(QLabel(trans._('color mode:')), 7, 0, 1, 1)
-        self.grid_layout.addWidget(self.colorModeComboBox, 7, 1, 1, 3)
-        self.grid_layout.addWidget(QLabel(trans._('contour:')), 8, 0, 1, 1)
-        self.grid_layout.addWidget(self.contourSpinBox, 8, 1, 1, 1)
-        self.grid_layout.addWidget(QLabel(trans._('n edit dim:')), 9, 0, 1, 1)
-        self.grid_layout.addWidget(self.ndimSpinBox, 9, 1, 1, 1)
-        self.grid_layout.addWidget(QLabel(trans._('contiguous:')), 10, 0, 1, 1)
-        self.grid_layout.addWidget(self.contigCheckBox, 10, 1, 1, 1)
-        self.grid_layout.addWidget(
-            QLabel(trans._('preserve\nlabels:')), 11, 0, 1, 2
+        self.layout().addRow(button_row)
+        self.layout().addRow(trans._('label:'), color_layout)
+        self.layout().addRow(self.opacityLabel, self.opacitySlider)
+        self.layout().addRow(trans._('brush size:'), self.brushSizeSlider)
+        self.layout().addRow(trans._('blending:'), self.blendComboBox)
+        self.layout().addRow(self.renderLabel, self.renderComboBox)
+        self.layout().addRow(trans._('color mode:'), self.colorModeComboBox)
+        self.layout().addRow(trans._('contour:'), self.contourSpinBox)
+        self.layout().addRow(trans._('n edit dim:'), self.ndimSpinBox)
+        self.layout().addRow(trans._('contiguous:'), self.contigCheckBox)
+        self.layout().addRow(
+            trans._('preserve\nlabels:'), self.preserveLabelsCheckBox
         )
-        self.grid_layout.addWidget(self.preserveLabelsCheckBox, 11, 1, 1, 1)
-        self.grid_layout.addWidget(
-            QLabel(trans._('show\nselected:')), 11, 2, 1, 1
+        self.layout().addRow(
+            trans._('show\nselected:'), self.selectedColorCheckbox
         )
-        self.grid_layout.addWidget(self.selectedColorCheckbox, 11, 3, 1, 1)
-        self.grid_layout.setRowStretch(12, 1)
-        self.grid_layout.setColumnStretch(1, 1)
-        self.grid_layout.setSpacing(4)
 
     def _on_mode_change(self, event):
         """Receive layer model mode change event and update checkbox ticks.
@@ -319,9 +300,7 @@ class QtLabelsControls(QtLayerControls):
             self.fill_button.setChecked(True)
         elif mode == Mode.ERASE:
             self.erase_button.setChecked(True)
-        elif mode == Mode.TRANSFORM:
-            pass
-        else:
+        elif mode != Mode.TRANSFORM:
             raise ValueError(trans._("Mode not recognized"))
 
     def changeRendering(self, text):
@@ -359,10 +338,7 @@ class QtLabelsControls(QtLayerControls):
         self.setFocus()
 
     def toggle_selected_mode(self, state):
-        if state == Qt.Checked:
-            self.layer.show_selected_label = True
-        else:
-            self.layer.show_selected_label = False
+        self.layer.show_selected_label = state == Qt.CheckState.Checked
 
     def changeSize(self, value):
         """Change paint brush size.
@@ -382,10 +358,7 @@ class QtLabelsControls(QtLayerControls):
         state : QCheckBox
             Checkbox indicating if labels are contiguous.
         """
-        if state == Qt.Checked:
-            self.layer.contiguous = True
-        else:
-            self.layer.contiguous = False
+        self.layer.contiguous = state == Qt.CheckState.Checked
 
     def change_n_edit_dim(self, value):
         """Change the number of editable dimensions of label layer.
@@ -419,7 +392,7 @@ class QtLabelsControls(QtLayerControls):
         state : QCheckBox
             Checkbox indicating if overwriting label is enabled.
         """
-        self.layer.preserve_labels = state == Qt.Checked
+        self.layer.preserve_labels = state == Qt.CheckState.Checked
 
     def change_color_mode(self):
         """Change color mode of label layer"""
@@ -496,7 +469,7 @@ class QtLabelsControls(QtLayerControls):
         """Receive layer model rendering change event and update dropdown menu."""
         with self.layer.events.rendering.blocker():
             index = self.renderComboBox.findText(
-                self.layer.rendering, Qt.MatchFixedString
+                self.layer.rendering, Qt.MatchFlag.MatchFixedString
             )
             self.renderComboBox.setCurrentIndex(index)
 
@@ -533,14 +506,17 @@ class QtColorBox(QWidget):
             self._on_selected_label_change
         )
         self.layer.events.opacity.connect(self._on_opacity_change)
+        self.layer.events.colormap.connect(self._on_colormap_change)
 
-        self.setAttribute(Qt.WA_DeleteOnClose)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
 
         self._height = 24
         self.setFixedWidth(self._height)
         self.setFixedHeight(self._height)
         self.setToolTip(trans._('Selected label color'))
 
+        self.color = None
+
     def _on_selected_label_change(self):
         """Receive layer model label selection change event & update colorbox."""
         self.update()
@@ -549,6 +525,10 @@ class QtColorBox(QWidget):
         """Receive layer model label selection change event & update colorbox."""
         self.update()
 
+    def _on_colormap_change(self):
+        """Receive label colormap change event & update colorbox."""
+        self.update()
+
     def paintEvent(self, event):
         """Paint the colorbox.  If no color, display a checkerboard pattern.
 
@@ -559,6 +539,7 @@ class QtColorBox(QWidget):
         """
         painter = QPainter(self)
         if self.layer._selected_color is None:
+            self.color = None
             for i in range(self._height // 4):
                 for j in range(self._height // 4):
                     if (i % 2 == 0 and j % 2 == 0) or (
@@ -576,6 +557,7 @@ class QtColorBox(QWidget):
             painter.setPen(QColor(*list(color)))
             painter.setBrush(QColor(*list(color)))
             painter.drawRect(0, 0, self._height, self._height)
+            self.color = tuple(color)
 
     def deleteLater(self):
         disconnect_events(self.layer.events, self)
