@@ -1,3 +1,4 @@
+import contextlib
 from typing import TYPE_CHECKING, Callable, List, Union
 
 from qtpy.QtWidgets import QAction, QMenu
@@ -125,18 +126,14 @@ class NapariMenu(QMenu):
         for ax in self.actions():
             ax.setData(None)
 
-            try:
+            with contextlib.suppress(AttributeError):
                 ax._destroy()
-            except AttributeError:
-                pass
-
         if self in self._INSTANCES:
             self._INSTANCES.remove(self)
 
     def update(self, event=None):
         """Update action enabled/disabled state based on action data."""
         for ax in self.actions():
-            data = ax.data()
-            if data:
+            if data := ax.data():
                 enabled_func = data.get('enabled', lambda event: True)
                 ax.setEnabled(bool(enabled_func(event)))
