@@ -2,6 +2,8 @@
 Custom Qt widgets that serve as native objects that the public-facing elements
 wrap.
 """
+
+import contextlib
 import inspect
 import os
 import sys
@@ -16,6 +18,8 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
+    Union,
+    cast,
 )
 from weakref import WeakValueDictionary
 
@@ -31,6 +35,7 @@ from qtpy.QtWidgets import (
     QToolTip,
     QWidget,
 )
+from superqt.utils import QSignalThrottler
 
 from ..plugins import menu_item_template as plugin_menu_item_template
 from ..plugins import plugin_manager
@@ -43,10 +48,11 @@ from ..utils.notifications import Notification
 from ..utils.theme import _themes, get_system_theme
 from ..utils.translations import trans
 from . import menus
+from .dialogs.confirm_close_dialog import ConfirmCloseDialog
 from .dialogs.qt_activity_dialog import QtActivityDialog
 from .dialogs.qt_notification import NapariQtNotification
 from .qt_event_loop import NAPARI_ICON_PATH, get_app, quit_app
-from .qt_resources import get_stylesheet, register_napari_themes
+from .qt_resources import get_stylesheet
 from .qt_viewer import QtViewer
 from .utils import QImg2array, qbytearray_to_str, str_to_qbytearray
 from .widgets.qt_viewer_dock_widget import (
@@ -58,6 +64,7 @@ from .widgets.qt_viewer_status_bar import ViewerStatusBar
 _sentinel = object()
 
 if TYPE_CHECKING:
+    from magicgui.widgets import Widget
     from qtpy.QtGui import QImage
 
     from ..viewer import Viewer
@@ -82,7 +89,7 @@ class _QtMainWindow(QMainWindow):
         self._quit_app = False
 
         self.setWindowIcon(QIcon(self._window_icon))
-        self.setAttribute(Qt.WA_DeleteOnClose)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
         self.setUnifiedTitleAndToolBarOnMac(True)
         center = QWidget(self)
         center.setLayout(QHBoxLayout())
@@ -126,11 +133,25 @@ class _QtMainWindow(QMainWindow):
                 self._qt_viewer.canvas._backend.screen_changed
             )
 
+        self.status_throttler = QSignalThrottler(parent=self)
+        self.status_throttler.setTimeout(50)
+
+        # In the GUI we expect lots of changes to the cursor position, so
+        # replace the direct connection with a throttled one.
+        with contextlib.suppress(IndexError):
+            viewer.cursor.events.position.disconnect(
+                viewer._update_status_bar_from_cursor
+            )
+        viewer.cursor.events.position.connect(self.status_throttler.throttle)
+        self.status_throttler.triggered.connect(
+            viewer._update_status_bar_from_cursor
+        )
+
     def statusBar(self) -> 'ViewerStatusBar':
         return super().statusBar()
 
     @classmethod
-    def current(cls):
+    def current(cls) -> Optional['_QtMainWindow']:
         return cls._instances[-1] if cls._instances else None
 
     @classmethod
@@ -138,27 +159,28 @@ class _QtMainWindow(QMainWindow):
         window = cls.current()
         return window._qt_viewer.viewer if window else None
 
-    def event(self, e):
+    def event(self, e: QEvent) -> bool:
         if (
-            e.type() == QEvent.ToolTip
+            e.type() == QEvent.Type.ToolTip
             and self._qt_viewer.viewer.tooltip.visible
         ):
-            QToolTip.showText(
-                e.globalPos(), self._qt_viewer.viewer.tooltip.text, self
+            # globalPos is for Qt5 e.globalPosition().toPoint() is for QT6
+            # https://doc-snapshots.qt.io/qt6-dev/qmouseevent-obsolete.html#globalPos
+            pnt = (
+                e.globalPosition().toPoint()
+                if hasattr(e, "globalPosition")
+                else e.globalPos()
             )
-        if e.type() == QEvent.Close:
+            QToolTip.showText(pnt, self._qt_viewer.viewer.tooltip.text, self)
+        if e.type() == QEvent.Type.Close:
             # when we close the MainWindow, remove it from the instances list
-            try:
+            with contextlib.suppress(ValueError):
                 _QtMainWindow._instances.remove(self)
-            except ValueError:
-                pass
-        if e.type() in {QEvent.WindowActivate, QEvent.ZOrderChange}:
+        if e.type() in {QEvent.Type.WindowActivate, QEvent.Type.ZOrderChange}:
             # upon activation or raise_, put window at the end of _instances
-            try:
+            with contextlib.suppress(ValueError):
                 inst = _QtMainWindow._instances
                 inst.append(inst.pop(inst.index(self)))
-            except ValueError:
-                pass
         return super().event(e)
 
     def _load_window_settings(self):
@@ -174,7 +196,7 @@ class _QtMainWindow(QMainWindow):
             window_position = (self.x(), self.y())
         else:
             width, height = window_position
-            screen_geo = QApplication.desktop().geometry()
+            screen_geo = QApplication.primaryScreen().geometry()
             if screen_geo.width() < width or screen_geo.height() < height:
                 window_position = (self.x(), self.y())
 
@@ -221,7 +243,7 @@ class _QtMainWindow(QMainWindow):
         Symmetric to the 'get_window_settings' accessor.
         """
         self.setUpdatesEnabled(False)
-        self.setWindowState(Qt.WindowNoState)
+        self.setWindowState(Qt.WindowState.WindowNoState)
 
         if window_position:
             window_position = QPoint(*window_position)
@@ -240,10 +262,10 @@ class _QtMainWindow(QMainWindow):
             self._qt_viewer.dockConsole.setVisible(False)
 
         if window_fullscreen:
-            self.setWindowState(Qt.WindowFullScreen)
+            self.setWindowState(Qt.WindowState.WindowFullScreen)
             self._maximized_flag = window_maximized
         elif window_maximized:
-            self.setWindowState(Qt.WindowMaximized)
+            self.setWindowState(Qt.WindowState.WindowMaximized)
 
         self.setUpdatesEnabled(True)
 
@@ -270,10 +292,19 @@ class _QtMainWindow(QMainWindow):
         if settings.application.save_window_state:
             settings.application.window_state = window_state
 
-    def close(self, quit_app=False):
+    def close(self, quit_app=False, confirm_need=False):
         """Override to handle closing app or just the window."""
-        self._quit_app = quit_app
-        return super().close()
+        if hasattr(self.status_throttler, "_timer"):
+            self.status_throttler._timer.stop()
+        if not quit_app and not self._qt_viewer.viewer.layers:
+            return super().close()
+        if (
+            not confirm_need
+            or not get_settings().application.confirm_close_window
+            or ConfirmCloseDialog(self, quit_app).exec_() == QDialog.Accepted
+        ):
+            self._quit_app = quit_app
+            return super().close()
 
     def close_window(self):
         """Close active dialog or active window."""
@@ -289,18 +320,19 @@ class _QtMainWindow(QMainWindow):
 
             try:
                 parent = parent.parent()
-            except Exception:
+            except AttributeError:
                 parent = getattr(parent, "_parent", None)
 
     def show(self, block=False):
         super().show()
+        self._qt_viewer.setFocus()
         if block:
             self._ev = QEventLoop()
             self._ev.exec()
 
     def changeEvent(self, event):
         """Handle window state changes."""
-        if event.type() == QEvent.WindowStateChange:
+        if event.type() == QEvent.Type.WindowStateChange:
             # TODO: handle maximization issue. When double clicking on the
             # title bar on Mac the resizeEvent is called an varying amount
             # of times which makes it hard to track the original size before
@@ -343,12 +375,21 @@ class _QtMainWindow(QMainWindow):
 
         Regardless of whether cmd Q, cmd W, or the close button is used...
         """
+        if (
+            event.spontaneous()
+            and get_settings().application.confirm_close_window
+            and self._qt_viewer.viewer.layers
+            and ConfirmCloseDialog(self, False).exec_() != QDialog.Accepted
+        ):
+            event.ignore()
+            return
+
         if self._ev and self._ev.isRunning():
             self._ev.quit()
 
         # Close any floating dockwidgets
         for dock in self.findChildren(QtViewerDockWidget):
-            if dock.isFloating():
+            if isinstance(dock, QWidget) and dock.isFloating():
                 dock.setFloating(False)
 
         self._save_current_window_settings()
@@ -358,7 +399,7 @@ class _QtMainWindow(QMainWindow):
         # test to complete its draw cycle, then pop back out of fullscreen.
         if self.isFullScreen():
             self.showNormal()
-            for _i in range(5):
+            for _ in range(5):
                 time.sleep(0.1)
                 QApplication.processEvents()
 
@@ -401,8 +442,7 @@ class Window:
         Help menu.
     main_menu : qtpy.QtWidgets.QMainWindow.menuBar
         Main menubar.
-    qt_viewer : QtViewer
-        Contained viewer widget.
+
     view_menu : qtpy.QtWidgets.QMenu
         View menu.
     window_menu : qtpy.QtWidgets.QMenu
@@ -425,7 +465,6 @@ class Window:
         # connect theme events before collecting plugin-provided themes
         # to ensure icons from the plugins are generated correctly.
         _themes.events.added.connect(self._add_theme)
-        _themes.events.added.connect(register_napari_themes)
         _themes.events.removed.connect(self._remove_theme)
 
         # discover any themes provided by plugins
@@ -452,11 +491,11 @@ class Window:
                 self._qt_viewer.dockPerformance, menu=self.window_menu
             )
 
-        viewer.events.status.connect(self._status_changed)
         viewer.events.help.connect(self._help_changed)
         viewer.events.title.connect(self._title_changed)
         viewer.events.theme.connect(self._update_theme)
         viewer.layers.events.connect(self.file_menu.update)
+        viewer.events.status.connect(self._status_changed)
 
         if show:
             self.show()
@@ -467,7 +506,7 @@ class Window:
                     self._qt_viewer.dockLayerList,
                 ],
                 [self._qt_viewer.dockLayerControls.minimumHeight(), 10000],
-                Qt.Vertical,
+                Qt.Orientation.Vertical,
             )
 
     def _setup_existing_themes(self, connect: bool = True):
@@ -496,7 +535,7 @@ class Window:
         theme.events.text.connect(self._update_theme_no_event)
         theme.events.warning.connect(self._update_theme_no_event)
         theme.events.current.connect(self._update_theme_no_event)
-        theme.events.icon.connect(self._theme_icon_changed)
+        theme.events.icon.connect(self._update_theme_no_event)
         theme.events.canvas.connect(
             lambda _: self._qt_viewer.canvas._set_theme_change(
                 get_settings().appearance.theme
@@ -520,7 +559,7 @@ class Window:
         theme.events.text.disconnect(self._update_theme_no_event)
         theme.events.warning.disconnect(self._update_theme_no_event)
         theme.events.current.disconnect(self._update_theme_no_event)
-        theme.events.icon.disconnect(self._theme_icon_changed)
+        theme.events.icon.disconnect(self._update_theme_no_event)
         theme.events.canvas.disconnect(
             lambda _: self._qt_viewer.canvas._set_theme_change(
                 get_settings().appearance.theme
@@ -546,22 +585,6 @@ class Window:
         theme = event.value
         self._disconnect_theme(theme)
 
-    def _theme_icon_changed(self):
-        """Trigger rebuild of theme and all resources.
-
-        This is really only required whenever there are changes to the `icon`
-        attribute on the `Theme` model. Most other attributes simply update
-        the stylesheet.
-        """
-        from .._qt.qt_resources import (
-            _register_napari_resources,
-            _unregister_napari_resources,
-        )
-
-        _unregister_napari_resources()
-        _register_napari_resources(True, force_rebuild=True)
-        self._update_theme()
-
     @property
     def qt_viewer(self):
         warnings.warn(
@@ -625,24 +648,16 @@ class Window:
         menubar is visible.
         """
         self.main_menu.setVisible(not self.main_menu.isVisible())
-        self._main_menu_shortcut.setEnabled(self.main_menu.isVisible())
+        self._main_menu_shortcut.setEnabled(not self.main_menu.isVisible())
 
-    def _tooltip_visibility_toggle(self, value):
-        get_settings().appearance.layer_tooltip_visibility = value
-
-    def _tooltip_visibility_toggled(self, event):
-        self.tooltip_menu.setChecked(
-            get_settings().appearance.layer_tooltip_visibility
-        )
-
-    def _toggle_fullscreen(self, event=None):
+    def _toggle_fullscreen(self):
         """Toggle fullscreen mode."""
         if self._qt_window.isFullScreen():
             self._qt_window.showNormal()
         else:
             self._qt_window.showFullScreen()
 
-    def _toggle_play(self, state=None):
+    def _toggle_play(self):
         """Toggle play."""
         if self._qt_viewer.dims.is_playing:
             self._qt_viewer.dims.stop()
@@ -651,7 +666,10 @@ class Window:
             self._qt_viewer.dims.play(axis)
 
     def add_plugin_dock_widget(
-        self, plugin_name: str, widget_name: str = None
+        self,
+        plugin_name: str,
+        widget_name: str = None,
+        tabify: bool = False,
     ) -> Tuple[QtViewerDockWidget, Any]:
         """Add plugin dock widget if not already added.
 
@@ -675,14 +693,14 @@ class Window:
         Widget = None
         dock_kwargs = {}
 
-        result = _npe2.get_widget_contribution(plugin_name, widget_name)
-        if result:
+        if result := _npe2.get_widget_contribution(plugin_name, widget_name):
             Widget, widget_name = result
 
         if Widget is None:
             Widget, dock_kwargs = plugin_manager.get_widget(
                 plugin_name, widget_name
             )
+
         if not widget_name:
             # if widget_name wasn't provided, `get_widget` will have
             # ensured that there is a single widget available.
@@ -696,11 +714,15 @@ class Window:
                 wdg = wdg._magic_widget
             return dock_widget, wdg
 
-        wdg = _instantiate_dock_widget(Widget, self._qt_viewer.viewer)
+        wdg = _instantiate_dock_widget(
+            Widget, cast('Viewer', self._qt_viewer.viewer)
+        )
 
         # Add dock widget
         dock_kwargs.pop('name', None)
-        dock_widget = self.add_dock_widget(wdg, name=full_name, **dock_kwargs)
+        dock_widget = self.add_dock_widget(
+            wdg, name=full_name, tabify=tabify, **dock_kwargs
+        )
         return dock_widget, wdg
 
     def _add_plugin_function_widget(self, plugin_name: str, widget_name: str):
@@ -728,7 +750,7 @@ class Window:
 
     def add_dock_widget(
         self,
-        widget: QWidget,
+        widget: Union[QWidget, 'Widget'],
         *,
         name: str = '',
         area: str = 'right',
@@ -736,6 +758,7 @@ class Window:
         shortcut=_sentinel,
         add_vertical_stretch=True,
         menu=None,
+        tabify: bool = False,
     ):
         """Convenience method to add a QDockWidget to the main window.
 
@@ -774,11 +797,8 @@ class Window:
             `dock_widget` that can pass viewer events.
         """
         if not name:
-            try:
+            with contextlib.suppress(AttributeError):
                 name = widget.objectName()
-            except AttributeError:
-                pass
-
             name = name or trans._(
                 "Dock widget {number}",
                 number=self._unnamed_dockwidget_count,
@@ -811,7 +831,7 @@ class Window:
                 add_vertical_stretch=add_vertical_stretch,
             )
 
-        self._add_viewer_dock_widget(dock_widget, menu=menu)
+        self._add_viewer_dock_widget(dock_widget, menu=menu, tabify=tabify)
 
         if hasattr(widget, 'reset_choices'):
             # Keep the dropdown menus in the widget in sync with the layer model
@@ -861,7 +881,9 @@ class Window:
                 _wdg = current_dws_in_area + [dock_widget]
                 # add sizes to push lower widgets up
                 sizes = list(range(1, len(_wdg) * 4, 4))
-                self._qt_window.resizeDocks(_wdg, sizes, Qt.Vertical)
+                self._qt_window.resizeDocks(
+                    _wdg, sizes, Qt.Orientation.Vertical
+                )
 
         if menu:
             action = dock_widget.toggleViewAction()
@@ -908,6 +930,7 @@ class Window:
             return
 
         if not isinstance(widget, QDockWidget):
+            dw: QDockWidget
             for dw in self._qt_window.findChildren(QDockWidget):
                 if dw.widget() is widget:
                     _dw: QDockWidget = dw
@@ -1019,6 +1042,39 @@ class Window:
         """
         self._qt_window.resize(width, height)
 
+    def set_geometry(self, left, top, width, height):
+        """Set the geometry of the widget
+
+        Parameters
+        ----------
+        left : int
+            X coordinate of the upper left border.
+        top : int
+            Y coordinate of the upper left border.
+        width : int
+            Width of the rectangle shape of the window.
+        height : int
+            Height of the rectangle shape of the window.
+        """
+        self._qt_window.setGeometry(left, top, width, height)
+
+    def geometry(self) -> Tuple[int, int, int, int]:
+        """Get the geometry of the widget
+
+        Returns
+        -------
+        left : int
+            X coordinate of the upper left border.
+        top : int
+            Y coordinate of the upper left border.
+        width : int
+            Width of the rectangle shape of the window.
+        height : int
+            Height of the rectangle shape of the window.
+        """
+        rect = self._qt_window.geometry()
+        return rect.left(), rect.top(), rect.width(), rect.height()
+
     def show(self, *, block=False):
         """Resize, show, and bring forward the window.
 
@@ -1096,21 +1152,19 @@ class Window:
     def _update_theme(self, event=None):
         """Update widget color theme."""
         settings = get_settings()
-        try:
+        with contextlib.suppress(AttributeError, RuntimeError):
             if event:
                 value = event.value
                 self._qt_viewer.viewer.theme = value
                 settings.appearance.theme = value
             else:
-                if settings.appearance.theme == "system":
-                    value = get_system_theme()
-                else:
-                    value = self._qt_viewer.viewer.theme
+                value = (
+                    get_system_theme()
+                    if settings.appearance.theme == "system"
+                    else self._qt_viewer.viewer.theme
+                )
 
             self._qt_window.setStyleSheet(get_stylesheet(value))
-        except (AttributeError, RuntimeError):
-            # wrapped C/C++ object may have been deleted?
-            pass
 
     def _status_changed(self, event):
         """Update status bar.
@@ -1120,7 +1174,16 @@ class Window:
         event : napari.utils.event.Event
             The napari event that triggered this method.
         """
-        self._status_bar.showMessage(event.value)
+        if isinstance(event.value, str):
+            self._status_bar.setStatusText(event.value)
+        else:
+            status_info = event.value
+            self._status_bar.setStatusText(
+                layer_base=status_info['layer_base'],
+                source_type=status_info['source_type'],
+                plugin=status_info['plugin'],
+                coordinates=status_info['coordinates'],
+            )
 
     def _title_changed(self, event):
         """Update window title.
@@ -1179,16 +1242,20 @@ class Window:
             if size is not None:
                 if len(size) != 2:
                     raise ValueError(
-                        f'screenshot size must be 2 values, got {len(size)}'
+                        trans._(
+                            'screenshot size must be 2 values, got {len_size}',
+                            len_size=len(size),
+                        )
                     )
                 # Scale the requested size to account for HiDPI
                 size = tuple(
-                    dim / self._qt_window.devicePixelRatio() for dim in size
+                    int(dim / self._qt_window.devicePixelRatio())
+                    for dim in size
                 )
                 canvas.size = size[::-1]  # invert x ad y for vispy
             if scale is not None:
                 # multiply canvas dimensions by the scale factor to get new size
-                canvas.size = tuple(dim * scale for dim in canvas.size)
+                canvas.size = tuple(int(dim * scale) for dim in canvas.size)
             try:
                 img = self._qt_viewer.canvas.native.grabFramebuffer()
                 if flash:
@@ -1257,14 +1324,11 @@ class Window:
         """Carry out various teardown tasks such as event disconnection."""
         self._setup_existing_themes(False)
         _themes.events.added.disconnect(self._add_theme)
-        _themes.events.added.disconnect(register_napari_themes)
         _themes.events.removed.disconnect(self._remove_theme)
         self._qt_viewer.viewer.layers.events.disconnect(self.file_menu.update)
         for menu in self.file_menu._INSTANCES:
-            try:
+            with contextlib.suppress(RuntimeError):
                 menu._destroy()
-            except RuntimeError:
-                pass
 
     def close(self):
         """Close the viewer window and cleanup sub-widgets."""
