@@ -1,9 +1,10 @@
 from __future__ import annotations
 
 import logging
-import os
+import traceback
 import warnings
-from typing import TYPE_CHECKING, List, Optional, Sequence, Tuple
+from pathlib import Path
+from typing import TYPE_CHECKING, List, Optional, Sequence, Tuple, Union
 from weakref import WeakSet
 
 import numpy as np
@@ -11,14 +12,17 @@ from qtpy.QtCore import QCoreApplication, QObject, Qt
 from qtpy.QtGui import QCursor, QGuiApplication
 from qtpy.QtWidgets import QFileDialog, QSplitter, QVBoxLayout, QWidget
 
+from napari_builtins.io import imsave_extensions
+
 from ..components._interaction_box_mouse_bindings import (
     InteractionBoxMouseBindings,
 )
 from ..components.camera import Camera
 from ..components.layerlist import LayerList
+from ..errors import MultipleReaderError, ReaderPluginError
 from ..layers.base.base import Layer
 from ..plugins import _npe2
-from ..plugins.utils import get_potential_readers
+from ..settings import get_settings
 from ..utils import config, perf
 from ..utils._proxies import ReadOnlyWrapper
 from ..utils.action_manager import action_manager
@@ -42,10 +46,7 @@ from ..utils.misc import in_ipython
 from ..utils.theme import get_theme
 from ..utils.translations import trans
 from .containers import QtLayerList
-from .dialogs.qt_reader_dialog import (
-    QtReaderDialog,
-    get_reader_choice_for_file,
-)
+from .dialogs.qt_reader_dialog import handle_gui_reading
 from .dialogs.screenshot_dialog import ScreenshotDialog
 from .perf.qt_performance import QtPerformance
 from .utils import QImg2array, circle_pixmap, crosshair_pixmap, square_pixmap
@@ -66,11 +67,10 @@ from .._vispy import (  # isort:skip
 
 
 if TYPE_CHECKING:
-    from ..components import ViewerModel
     from npe2.manifest.contributions import WriterContribution
 
-from ..settings import get_settings
-from ..utils.io import imsave_extensions
+    from ..components import ViewerModel
+    from .layer_controls import QtLayerControlsContainer
 
 
 def _npe2_decode_selected_filter(
@@ -164,8 +164,6 @@ class QtViewer(QSplitter):
         Dimension sliders; Qt View for Dims model.
     dockConsole : QtViewerDockWidget
         QWidget wrapped in a QDockWidget with forwarded viewer events.
-    aboutKeybindings : QtAboutKeybindings
-        Key bindings for the 'About' Qt dialog.
     dockLayerControls : QtViewerDockWidget
         QWidget wrapped in a QDockWidget with forwarded viewer events.
     dockLayerList : QtViewerDockWidget
@@ -187,12 +185,10 @@ class QtViewer(QSplitter):
     _instances = WeakSet()
 
     def __init__(self, viewer: ViewerModel, show_welcome_screen: bool = False):
-        # Avoid circular import.
-        from .layer_controls import QtLayerControlsContainer
 
         super().__init__()
         self._instances.add(self)
-        self.setAttribute(Qt.WA_DeleteOnClose)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
 
         self._show_welcome_screen = show_welcome_screen
 
@@ -202,60 +198,18 @@ class QtViewer(QSplitter):
 
         self.viewer = viewer
         self.dims = QtDims(self.viewer.dims)
-        self.controls = QtLayerControlsContainer(self.viewer)
-        self.layers = QtLayerList(self.viewer.layers)
-        self.layerButtons = QtLayerButtons(self.viewer)
-        self.viewerButtons = QtViewerButtons(self.viewer)
+        self._controls = None
+        self._layers = None
+        self._layersButtons = None
+        self._viewerButtons = None
         self._key_map_handler = KeymapHandler()
         self._key_map_handler.keymap_providers = [self.viewer]
         self._console = None
 
-        layerList = QWidget()
-        layerList.setObjectName('layerList')
-        layerListLayout = QVBoxLayout()
-        layerListLayout.addWidget(self.layerButtons)
-        layerListLayout.addWidget(self.layers)
-        layerListLayout.addWidget(self.viewerButtons)
-        layerListLayout.setContentsMargins(8, 4, 8, 6)
-        layerList.setLayout(layerListLayout)
-
-        self.dockLayerList = QtViewerDockWidget(
-            self,
-            layerList,
-            name=trans._('layer list'),
-            area='left',
-            allowed_areas=['left', 'right'],
-            object_name='layer list',
-            close_btn=False,
-        )
-        self.dockLayerControls = QtViewerDockWidget(
-            self,
-            self.controls,
-            name=trans._('layer controls'),
-            area='left',
-            allowed_areas=['left', 'right'],
-            object_name='layer controls',
-            close_btn=False,
-        )
-        self.dockConsole = QtViewerDockWidget(
-            self,
-            QWidget(),
-            name=trans._('console'),
-            area='bottom',
-            allowed_areas=['top', 'bottom'],
-            object_name='console',
-            close_btn=False,
-        )
-        self.dockConsole.setVisible(False)
-        # because the console is loaded lazily in the @getter, this line just
-        # gets (or creates) the console when the dock console is made visible.
-        self.dockConsole.visibilityChanged.connect(self._ensure_connect)
-        self.dockLayerControls.visibilityChanged.connect(self._constrain_width)
-        self.dockLayerList.setMaximumWidth(258)
-        self.dockLayerList.setMinimumWidth(258)
-
-        # Only created if using perfmon.
-        self.dockPerformance = self._create_performance_dock_widget()
+        self._dockLayerList = None
+        self._dockLayerControls = None
+        self._dockConsole = None
+        self._dockPerformance = None
 
         # This dictionary holds the corresponding vispy visual for each layer
         self.layer_to_visual = {}
@@ -266,23 +220,25 @@ class QtViewer(QSplitter):
         self._canvas_overlay = QtWidgetOverlay(self, self.canvas.native)
         self._canvas_overlay.set_welcome_visible(show_welcome_screen)
         self._canvas_overlay.sig_dropped.connect(self.dropEvent)
+        self._canvas_overlay.leave.connect(self._leave_canvas)
+        self._canvas_overlay.enter.connect(self._enter_canvas)
 
         main_widget = QWidget()
         main_layout = QVBoxLayout()
-        main_layout.setContentsMargins(10, 22, 10, 2)
+        main_layout.setContentsMargins(0, 2, 0, 2)
         main_layout.addWidget(self._canvas_overlay)
         main_layout.addWidget(self.dims)
-        main_layout.setSpacing(10)
+        main_layout.setSpacing(0)
         main_widget.setLayout(main_layout)
 
-        self.setOrientation(Qt.Vertical)
+        self.setOrientation(Qt.Orientation.Vertical)
         self.addWidget(main_widget)
 
         self._cursors = {
-            'cross': Qt.CrossCursor,
-            'forbidden': Qt.ForbiddenCursor,
-            'pointing': Qt.PointingHandCursor,
-            'standard': QCursor(),
+            'cross': Qt.CursorShape.CrossCursor,
+            'forbidden': Qt.CursorShape.ForbiddenCursor,
+            'pointing': Qt.CursorShape.PointingHandCursor,
+            'standard': Qt.CursorShape.ArrowCursor,
         }
 
         self._on_active_change()
@@ -300,9 +256,6 @@ class QtViewer(QSplitter):
 
         self.setAcceptDrops(True)
 
-        for layer in self.viewer.layers:
-            self._add_layer(layer)
-
         self.view = self.canvas.central_widget.add_view(border_width=0)
         self.camera = VispyCamera(
             self.view, self.viewer.camera, self.viewer.dims
@@ -334,6 +287,104 @@ class QtViewer(QSplitter):
         # bind shortcuts stored in settings last.
         self._bind_shortcuts()
 
+        for layer in self.viewer.layers:
+            self._add_layer(layer)
+
+    @property
+    def controls(self) -> QtLayerControlsContainer:
+        if self._controls is None:
+            # Avoid circular import.
+            from .layer_controls import QtLayerControlsContainer
+
+            self._controls = QtLayerControlsContainer(self.viewer)
+        return self._controls
+
+    @property
+    def layers(self) -> QtLayerList:
+        if self._layers is None:
+            self._layers = QtLayerList(self.viewer.layers)
+        return self._layers
+
+    @property
+    def layerButtons(self) -> QtLayerButtons:
+        if self._layersButtons is None:
+            self._layersButtons = QtLayerButtons(self.viewer)
+        return self._layersButtons
+
+    @property
+    def viewerButtons(self) -> QtViewerButtons:
+        if self._viewerButtons is None:
+            self._viewerButtons = QtViewerButtons(self.viewer)
+        return self._viewerButtons
+
+    @property
+    def dockLayerList(self) -> QtViewerDockWidget:
+        if self._dockLayerList is None:
+            layerList = QWidget()
+            layerList.setObjectName('layerList')
+            layerListLayout = QVBoxLayout()
+            layerListLayout.addWidget(self.layerButtons)
+            layerListLayout.addWidget(self.layers)
+            layerListLayout.addWidget(self.viewerButtons)
+            layerListLayout.setContentsMargins(8, 4, 8, 6)
+            layerList.setLayout(layerListLayout)
+            self._dockLayerList = QtViewerDockWidget(
+                self,
+                layerList,
+                name=trans._('layer list'),
+                area='left',
+                allowed_areas=['left', 'right'],
+                object_name='layer list',
+                close_btn=False,
+            )
+        return self._dockLayerList
+
+    @property
+    def dockLayerControls(self) -> QtViewerDockWidget:
+        if self._dockLayerControls is None:
+            self._dockLayerControls = QtViewerDockWidget(
+                self,
+                self.controls,
+                name=trans._('layer controls'),
+                area='left',
+                allowed_areas=['left', 'right'],
+                object_name='layer controls',
+                close_btn=False,
+            )
+        return self._dockLayerControls
+
+    @property
+    def dockConsole(self) -> QtViewerDockWidget:
+        if self._dockConsole is None:
+            self._dockConsole = QtViewerDockWidget(
+                self,
+                QWidget(),
+                name=trans._('console'),
+                area='bottom',
+                allowed_areas=['top', 'bottom'],
+                object_name='console',
+                close_btn=False,
+            )
+            self._dockConsole.setVisible(False)
+            self._dockConsole.visibilityChanged.connect(self._ensure_connect)
+        return self._dockConsole
+
+    @property
+    def dockPerformance(self) -> QtViewerDockWidget:
+        if self._dockPerformance is None:
+            self._dockPerformance = self._create_performance_dock_widget()
+        return self._dockPerformance
+
+    def _leave_canvas(self):
+        """disable status on canvas leave"""
+        self.viewer.status = ""
+        self.viewer.mouse_over_canvas = False
+
+    def _enter_canvas(self):
+        """enable status on canvas enter"""
+        self.viewer.status = "Ready"
+        self.viewer.mouse_over_canvas = True
+
     def _ensure_connect(self):
         # lazy load console
         id(self.console)
@@ -438,7 +489,7 @@ class QtViewer(QSplitter):
                     self.console.push(
                         {'napari': napari, 'action_manager': action_manager}
                     )
-            except ImportError:
+            except ModuleNotFoundError:
                 warnings.warn(
                     trans._(
                         'napari-console not found. It can be installed with'
@@ -446,6 +497,14 @@ class QtViewer(QSplitter):
                     )
                 )
                 self._console = None
+            except ImportError:
+                traceback.print_exc()
+                warnings.warn(
+                    trans._(
+                        'error importing napari-console. See console for full error.'
+                    )
+                )
+                self._console = None
         return self._console
 
     @console.setter
@@ -455,19 +514,6 @@ class QtViewer(QSplitter):
             self.dockConsole.setWidget(console)
             console.setParent(self.dockConsole)
 
-    def _constrain_width(self, event):
-        """Allow the layer controls to be wider, only if floated.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event
-            The napari event that triggered this method.
-        """
-        if self.dockLayerControls.isFloating():
-            self.controls.setMaximumWidth(700)
-        else:
-            self.controls.setMaximumWidth(220)
-
     def _on_active_change(self):
         """When active layer changes change keymap handler."""
         self._key_map_handler.keymap_providers = (
@@ -510,8 +556,8 @@ class QtViewer(QSplitter):
                 vispy_layer.events.loaded.connect(self._qt_poll.wake_up)
 
         vispy_layer.node.parent = self.view.scene
-        vispy_layer.order = len(self.viewer.layers) - 1
         self.layer_to_visual[layer] = vispy_layer
+        self._reorder_layers()
 
     def _remove_layer(self, event):
         """When a layer is removed, remove its parent.
@@ -580,8 +626,12 @@ class QtViewer(QSplitter):
             ),
         )
         logging.debug(
-            f'QFileDialog - filename: {filename or None} '
-            f'selected_filter: {selected_filter or None}'
+            trans._(
+                'QFileDialog - filename: {filename} '
+                'selected_filter: {selected_filter}',
+                filename=filename or None,
+                selected_filter=selected_filter or None,
+            )
         )
 
         if filename:
@@ -676,7 +726,7 @@ class QtViewer(QSplitter):
         if dial.exec_():
             update_save_history(dial.selectedFiles()[0])
 
-    def _open_files_dialog(self):
+    def _open_files_dialog(self, choose_plugin=False):
         """Add files from the menubar."""
         dlg = QFileDialog()
         hist = get_open_history()
@@ -694,10 +744,13 @@ class QtViewer(QSplitter):
         )
 
         if (filenames != []) and (filenames is not None):
-            self.viewer.open(filenames)
+            for filename in filenames:
+                self._qt_open(
+                    [filename], stack=False, choose_plugin=choose_plugin
+                )
             update_open_history(filenames[0])
 
-    def _open_files_dialog_as_stack_dialog(self):
+    def _open_files_dialog_as_stack_dialog(self, choose_plugin=False):
         """Add files as a stack, from the menubar."""
         dlg = QFileDialog()
         hist = get_open_history()
@@ -715,20 +768,20 @@ class QtViewer(QSplitter):
         )
 
         if (filenames != []) and (filenames is not None):
-            self.viewer.open(filenames, stack=True)
+            self._qt_open(filenames, stack=True, choose_plugin=choose_plugin)
             update_open_history(filenames[0])
 
-    def _open_folder_dialog(self):
+    def _open_folder_dialog(self, choose_plugin=False):
         """Add a folder of files from the menubar."""
         dlg = QFileDialog()
         hist = get_open_history()
         dlg.setHistory(hist)
 
         folder = dlg.getExistingDirectory(
-            parent=self,
-            caption=trans._('Select folder...'),
-            directory=hist[0],  # home dir by default
-            options=(
+            self,
+            trans._('Select folder...'),
+            hist[0],  # home dir by default
+            (
                 QFileDialog.DontUseNativeDialog
                 if in_ipython()
                 else QFileDialog.Options()
@@ -736,9 +789,64 @@ class QtViewer(QSplitter):
         )
 
         if folder not in {'', None}:
-            self.viewer.open([folder])
+            self._qt_open([folder], stack=False, choose_plugin=choose_plugin)
             update_open_history(folder)
 
+    def _qt_open(
+        self,
+        filenames: List[str],
+        stack: Union[bool, List[List[str]]],
+        choose_plugin: bool = False,
+        plugin: str = None,
+        layer_type: str = None,
+        **kwargs,
+    ):
+        """Open files, potentially popping reader dialog for plugin selection.
+
+        Call ViewerModel.open and catch errors that could
+        be fixed by user making a plugin choice.
+
+        Parameters
+        ----------
+        filenames : List[str]
+            paths to open
+        choose_plugin : bool
+            True if user wants to explicitly choose the plugin else False
+        stack : bool or list[list[str]]
+            whether to stack files or not. Can also be a list containing
+            files to stack.
+        plugin: str
+            plugin to use for reading
+        layer_type: str
+            layer type for opened layers
+        """
+        if choose_plugin:
+            handle_gui_reading(
+                filenames, self, stack, plugin_override=choose_plugin, **kwargs
+            )
+            return
+
+        try:
+            self.viewer.open(
+                filenames,
+                stack=stack,
+                plugin=plugin,
+                layer_type=layer_type,
+                **kwargs,
+            )
+        except ReaderPluginError as e:
+            handle_gui_reading(
+                filenames,
+                self,
+                stack,
+                e.reader_plugin,
+                e,
+                layer_type=layer_type,
+                **kwargs,
+            )
+        except MultipleReaderError:
+            handle_gui_reading(filenames, self, stack, **kwargs)
+
     def _toggle_chunk_outlines(self):
         """Toggle whether we are drawing outlines around the chunks."""
         from ..layers.image.experimental.octree_image import _OctreeImageBase
@@ -753,23 +861,24 @@ class QtViewer(QSplitter):
 
     def _on_cursor(self):
         """Set the appearance of the mouse cursor."""
-        cursor = self.viewer.cursor.style
-        # Scale size by zoom if needed
-        if self.viewer.cursor.scaled:
-            size = self.viewer.cursor.size * self.viewer.camera.zoom
-        else:
-            size = self.viewer.cursor.size
 
-        if cursor == 'square':
+        cursor = self.viewer.cursor.style
+        if cursor in {'square', 'circle'}:
+
+            # Scale size by zoom if needed
+            size = self.viewer.cursor.size
+            if self.viewer.cursor.scaled:
+                size *= self.viewer.camera.zoom
+
+            size = int(size)
+
             # make sure the square fits within the current canvas
-            if size < 8 or size > (
-                min(*self.viewer.window._qt_viewer.canvas.size) - 4
-            ):
+            if size < 8 or size > (min(*self.canvas.size) - 4):
                 q_cursor = self._cursors['cross']
+            elif cursor == 'circle':
+                q_cursor = QCursor(circle_pixmap(size))
             else:
                 q_cursor = QCursor(square_pixmap(size))
-        elif cursor == 'circle':
-            q_cursor = QCursor(circle_pixmap(size))
         elif cursor == 'crosshair':
             q_cursor = QCursor(crosshair_pixmap())
         else:
@@ -782,6 +891,9 @@ class QtViewer(QSplitter):
 
         Imports the console the first time it is requested.
         """
+        if in_ipython():
+            return
+
         # force instantiation of console if not already instantiated
         _ = self.console
 
@@ -823,6 +935,11 @@ class QtViewer(QSplitter):
         mapped_position = transform.imap(list(position))[:nd]
         position_world_slice = mapped_position[::-1]
 
+        # handle position for 3D views of 2D data
+        nd_point = len(self.viewer.dims.point)
+        if nd_point < nd:
+            position_world_slice = position_world_slice[-nd_point:]
+
         position_world = list(self.viewer.dims.point)
         for i, d in enumerate(self.viewer.dims.displayed):
             position_world[d] = position_world_slice[i]
@@ -1026,14 +1143,21 @@ class QtViewer(QSplitter):
 
         Parameters
         ----------
-        event : qtpy.QtCore.QEvent
+        event : qtpy.QtCore.QDragEvent
             Event from the Qt context.
         """
         if event.mimeData().hasUrls():
+            self._set_drag_status()
             event.accept()
         else:
             event.ignore()
 
+    def _set_drag_status(self):
+        """Set dedicated status message when dragging files into viewer"""
+        self.viewer.status = trans._(
+            'Hold <Alt> key to open plugin selection. Hold <Shift> to open files as stack.'
+        )
+
     def dropEvent(self, event):
         """Add local files and web URLS with drag and drop.
 
@@ -1045,143 +1169,43 @@ class QtViewer(QSplitter):
 
         Parameters
         ----------
-        event : qtpy.QtCore.QEvent
+        event : qtpy.QtCore.QDropEvent
             Event from the Qt context.
         """
-        shift_down = QGuiApplication.keyboardModifiers() & Qt.ShiftModifier
+        shift_down = (
+            QGuiApplication.keyboardModifiers()
+            & Qt.KeyboardModifier.ShiftModifier
+        )
+        alt_down = (
+            QGuiApplication.keyboardModifiers()
+            & Qt.KeyboardModifier.AltModifier
+        )
         filenames = []
         for url in event.mimeData().urls():
             if url.isLocalFile():
-                filenames.append(url.toLocalFile())
+                # directories get a trailing "/", Path conversion removes it
+                filenames.append(str(Path(url.toLocalFile())))
             else:
                 filenames.append(url.toString())
 
-        # if trying to open as a stack, open with any available reader
-        if shift_down:
-            self.viewer.open(filenames, stack=bool(shift_down))
-            return
-
-        for filename in filenames:
-            # get available readers for this file from all registered plugins
-            readers = get_potential_readers(filename)
-            if not readers:
-                warnings.warn(
-                    trans._(
-                        'No readers found to try reading {filename}.',
-                        deferred=True,
-                        filename=filename,
-                    )
-                )
-                continue
-
-            # see whether an existing setting can be used
-            _, extension = os.path.splitext(filename)
-            error_message = self._try_reader_from_settings(
-                readers, extension, filename
-            )
-            # we've successfully opened file, move to the next one
-            if error_message is None:
-                continue
-
-            # there is no existing setting, or it failed, get choice from user
-            readerDialog = QtReaderDialog(
-                parent=self,
-                pth=filename,
-                extension=extension,
-                error_message=error_message,
-                readers=readers,
-            )
-            self._get_and_try_preferred_reader(
-                readerDialog, readers, error_message
-            )
-
-    def _try_reader_from_settings(self, readers, extension, filename):
-        """Read settings and try to open file with preferred reader
-
-        Returns None when file was successfully opened with preferred reader,
-        otherwise returns appropriate error message.
-
-        Parameters
-        ----------
-        readers : Dict[str, str]
-            dictionary of display_name:plugin_name for potential readers
-        extension : str
-            file extension of the given filename
-        filename : str
-            path to file trying to open
-
-        Returns
-        -------
-        Optional[str]
-            return None when file was successfully opened, otherwise error message
-        """
-        reader_associations = get_settings().plugins.extension2reader
-        error_message = ''
-        # if we have an existing setting for this extension
-        if extension and extension in reader_associations:
-            display_name = reader_associations[extension]
-            # if this plugin is currently registered
-            if display_name in readers:
-                try:
-                    # try to open using the associated plugin
-                    self.viewer.open(
-                        filename,
-                        plugin=readers[display_name],
-                    )
-                    # we've opened file successfully, return
-                    return None
-                except Exception as e:
-                    error_message = f"Tried to open file with {display_name}, but reading failed ({e}).\n"
-            else:
-                error_message = f"Can't find {display_name} plugin associated with {extension} files.\n"
-        return error_message
-
-    def _get_and_try_preferred_reader(
-        self, readerDialog, readers, error_message
-    ):
-        """Get preferred reader from user through dialog and try to open file
-
-        Parameters
-        ----------
-        readerDialog : QtReaderDialog
-            dialog for user to select their preferences
-        readers : Dict[str, str]
-            dictionary of display_name:plugin_name of available readers
-        error_message : str
-            error message to show to user about failed reading attempts
-        """
-        # get plugin choice from user
-        # choice is None if file was opened or user chose cancel
-        choice = get_reader_choice_for_file(
-            readerDialog, readers, error_message
+        self._qt_open(
+            filenames,
+            stack=bool(shift_down),
+            choose_plugin=bool(alt_down),
         )
-        if choice:
-            display_name, persist_choice = choice
-            plugin_name = readers[display_name]
-            self.viewer.open(
-                readerDialog._current_file,
-                plugin=plugin_name,
-            )
-            # do we have settings to save?
-            if persist_choice:
-                # need explicit reassignment of object for persistence
-                get_settings().plugins.extension2reader = {
-                    **get_settings().plugins.extension2reader,
-                    readerDialog._extension: display_name,
-                }
 
     def closeEvent(self, event):
         """Cleanup and close.
 
         Parameters
         ----------
-        event : qtpy.QtCore.QEvent
+        event : qtpy.QtCore.QCloseEvent
             Event from the Qt context.
         """
         self.layers.close()
 
         # if the viewer.QtDims object is playing an axis, we need to terminate
-        # the AnimationThread before close, otherwise it will cauyse a segFault
+        # the AnimationThread before close, otherwise it will cause a segFault
         # or Abort trap. (calling stop() when no animation is occurring is also
         # not a problem)
         self.dims.stop()
