@@ -1,3 +1,4 @@
+import contextlib
 import re
 from collections import OrderedDict
 
@@ -18,6 +19,7 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+from vispy.util import keys
 
 from ...layers import Image, Labels, Points, Shapes, Surface, Vectors
 from ...settings import get_settings
@@ -75,11 +77,11 @@ class ShortcutEditor(QWidget):
 
         # Set up dictionary for layers and associated actions.
         all_actions = action_manager._actions.copy()
-        self.key_bindings_strs[self.VIEWER_KEYBINDINGS] = []
+        self.key_bindings_strs[self.VIEWER_KEYBINDINGS] = {}
 
         for layer in layers:
             if len(layer.class_keymap) == 0:
-                actions = []
+                actions = {}
             else:
                 actions = action_manager._get_layer_actions(layer)
                 for name, action in actions.items():
@@ -91,7 +93,7 @@ class ShortcutEditor(QWidget):
 
         # Widget set up
         self.layer_combo_box.addItems(list(self.key_bindings_strs))
-        self.layer_combo_box.activated[str].connect(self._set_table)
+        self.layer_combo_box.currentTextChanged.connect(self._set_table)
         self.layer_combo_box.setCurrentText(self.VIEWER_KEYBINDINGS)
         self._set_table()
         self._label.setText(trans._("Group"))
@@ -112,6 +114,13 @@ class ShortcutEditor(QWidget):
         layout = QVBoxLayout()
         layout.addLayout(hlayout2)
         layout.addWidget(self._table)
+        layout.addWidget(
+            QLabel(
+                trans._(
+                    "To edit, double-click the keybinding. To unbind a shortcut, use Backspace or Delete. To set Backspace or Delete, first unbind."
+                )
+            )
+        )
 
         self.setLayout(layout)
 
@@ -157,28 +166,23 @@ class ShortcutEditor(QWidget):
         self._action_name_col = 0
         self._icon_col = 1
         self._shortcut_col = 2
-        self._action_col = 3
+        self._shortcut_col2 = 3
+        self._action_col = 4
 
         # Set header strings for table.
-        header_strs = ['', '', '', '']
+        header_strs = ['', '', '', '', '']
         header_strs[self._action_name_col] = trans._('Action')
         header_strs[self._shortcut_col] = trans._('Keybinding')
+        header_strs[self._shortcut_col2] = trans._('Alternative Keybinding')
 
         # If no layer_str, then set the page to the viewer keybindings page.
-        if layer_str == '':
+        if not layer_str:
             layer_str = self.VIEWER_KEYBINDINGS
 
         # If rebuilding the table, then need to disconnect the connection made
         # previously as well as clear the table contents.
-        try:
+        with contextlib.suppress(TypeError, RuntimeError):
             self._table.cellChanged.disconnect(self._set_keybinding)
-        except TypeError:
-            # if building the first time, the cells are not yet connected so this would fail.
-            pass
-        except RuntimeError:
-            # Needed to pass some tests.
-            pass
-
         self._table.clearContents()
 
         # Table styling set up.
@@ -194,11 +198,14 @@ class ShortcutEditor(QWidget):
 
             # Set up table based on number of actions and needed columns.
             self._table.setRowCount(len(actions))
-            self._table.setColumnCount(4)
+            self._table.setColumnCount(5)
             # Set up delegate in order to capture keybindings.
             self._table.setItemDelegateForColumn(
                 self._shortcut_col, ShortcutDelegate(self._table)
             )
+            self._table.setItemDelegateForColumn(
+                self._shortcut_col2, ShortcutDelegate(self._table)
+            )
             self._table.setHorizontalHeaderLabels(header_strs)
             self._table.verticalHeader().setVisible(False)
 
@@ -208,6 +215,7 @@ class ShortcutEditor(QWidget):
             # Column set up.
             self._table.setColumnWidth(self._action_name_col, 250)
             self._table.setColumnWidth(self._shortcut_col, 200)
+            self._table.setColumnWidth(self._shortcut_col2, 200)
             self._table.setColumnWidth(self._icon_col, 50)
 
             # Go through all the actions in the layer and add them to the table.
@@ -216,13 +224,13 @@ class ShortcutEditor(QWidget):
                 shortcuts = action_manager._shortcuts.get(action_name, [])
                 # Set action description.  Make sure its not selectable/editable.
                 item = QTableWidgetItem(action.description)
-                item.setFlags(Qt.NoItemFlags)
+                item.setFlags(Qt.ItemFlag.NoItemFlags)
                 self._table.setItem(row, self._action_name_col, item)
 
                 # Create empty item in order to make sure this column is not
                 # selectable/editable.
                 item = QTableWidgetItem("")
-                item.setFlags(Qt.NoItemFlags)
+                item.setFlags(Qt.ItemFlag.NoItemFlags)
                 self._table.setItem(row, self._icon_col, item)
 
                 # Set the shortcuts in table.
@@ -231,6 +239,13 @@ class ShortcutEditor(QWidget):
                 )
                 self._table.setItem(row, self._shortcut_col, item_shortcut)
 
+                item_shortcut2 = QTableWidgetItem(
+                    Shortcut(list(shortcuts)[1]).platform
+                    if len(shortcuts) > 1
+                    else ""
+                )
+                self._table.setItem(row, self._shortcut_col2, item_shortcut2)
+
                 # action_name is stored in the table to use later, but is not shown on dialog.
                 item_action = QTableWidgetItem(action_name)
                 self._table.setItem(row, self._action_col, item_action)
@@ -246,9 +261,92 @@ class ShortcutEditor(QWidget):
 
             self._table.setColumnHidden(self._action_col, True)
             item = QTableWidgetItem(trans._('No key bindings'))
-            item.setFlags(Qt.NoItemFlags)
+            item.setFlags(Qt.ItemFlag.NoItemFlags)
             self._table.setItem(0, 0, item)
 
+    def _get_layer_actions(self):
+        current_layer_text = self.layer_combo_box.currentText()
+        layer_actions = self.key_bindings_strs[current_layer_text]
+        actions_all = layer_actions.copy()
+        if current_layer_text is not self.VIEWER_KEYBINDINGS:
+            viewer_actions = self.key_bindings_strs[self.VIEWER_KEYBINDINGS]
+
+            actions_all.update(viewer_actions)
+        return actions_all
+
+    def _restore_shortcuts(self, row):
+        action_name = self._table.item(row, self._action_col).text()
+        shortcuts = action_manager._shortcuts.get(action_name, [])
+        with lock_keybind_update(self):
+            self._table.item(row, self._shortcut_col).setText(
+                Shortcut(list(shortcuts)[0]).platform if shortcuts else ""
+            )
+            self._table.item(row, self._shortcut_col2).setText(
+                Shortcut(list(shortcuts)[1]).platform
+                if len(shortcuts) > 1
+                else ""
+            )
+
+    def _mark_conflicts(self, new_shortcut, row) -> bool:
+        # Go through all layer actions to determine if the new shortcut is already here.
+        current_action = self._table.item(row, self._action_col).text()
+        actions_all = self._get_layer_actions()
+        current_item = self._table.currentItem()
+        for row1, (action_name, action) in enumerate(actions_all.items()):
+            shortcuts = action_manager._shortcuts.get(action_name, [])
+
+            if new_shortcut not in shortcuts:
+                continue
+            # Shortcut is here (either same action or not), don't replace in settings.
+            if action_name != current_action:
+                # the shortcut is saved to a different action
+
+                # show warning symbols
+                self._show_warning_icons([row, row1])
+
+                # show warning message
+                message = trans._(
+                    "The keybinding <b>{new_shortcut}</b>  is already assigned to <b>{action_description}</b>; change or clear that shortcut before assigning <b>{new_shortcut}</b> to this one.",
+                    new_shortcut=new_shortcut,
+                    action_description=action.description,
+                )
+                self._show_warning(new_shortcut, action, row, message)
+
+                self._restore_shortcuts(row)
+
+                self._cleanup_warning_icons([row, row1])
+
+                return False
+
+            else:
+                # This shortcut was here.  Reformat and reset text.
+                format_shortcut = Shortcut(new_shortcut).platform
+                with lock_keybind_update(self):
+                    current_item.setText(format_shortcut)
+
+        return True
+
+    def _show_bind_shortcut_error(
+        self, current_action, current_shortcuts, row, new_shortcut
+    ):
+        action_manager._shortcuts[current_action] = []
+        # need to rebind the old shortcut
+        action_manager.unbind_shortcut(current_action)
+        for short in current_shortcuts:
+            action_manager.bind_shortcut(current_action, short)
+
+        # Show warning message to let user know this shortcut is invalid.
+        self._show_warning_icons([row])
+
+        message = trans._(
+            "<b>{new_shortcut}</b> is not a valid keybinding.",
+            new_shortcut=new_shortcut,
+        )
+        self._show_warning(new_shortcut, current_action, row, message)
+
+        self._cleanup_warning_icons([row])
+        self._restore_shortcuts(row)
+
     def _set_keybinding(self, row, col):
         """Checks the new keybinding to determine if it can be set.
 
@@ -260,13 +358,12 @@ class ShortcutEditor(QWidget):
             Column being edited (shortcut column).
         """
 
-        if self._skip is True:
-            # Do nothing if the text is setting to a symbol.
-            # Its already been handled.
-            self._skip = False
+        if self._skip:
             return
 
-        if col == self._shortcut_col:
+        self._table.setCurrentItem(self._table.item(row, col))
+
+        if col in {self._shortcut_col, self._shortcut_col2}:
             # Get all layer actions and viewer actions in order to determine
             # the new shortcut is not already set to an action.
 
@@ -283,133 +380,57 @@ class ShortcutEditor(QWidget):
             # get the current item from shortcuts column
             current_item = self._table.currentItem()
             new_shortcut = current_item.text()
-            new_shortcut = new_shortcut[0].upper() + new_shortcut[1:]
+            if new_shortcut:
+                new_shortcut = new_shortcut[0].upper() + new_shortcut[1:]
 
             # get the current action name
             current_action = self._table.item(row, self._action_col).text()
 
             # get the original shortcutS
             current_shortcuts = list(
-                action_manager._shortcuts.get(current_action, {})
+                action_manager._shortcuts.get(current_action, [])
             )
 
             # Flag to indicate whether to set the new shortcut.
-            replace = True
-            # Go through all layer actions to determine if the new shortcut is already here.
-            for row1, (action_name, action) in enumerate(actions_all.items()):
-                shortcuts = action_manager._shortcuts.get(action_name, [])
-
-                if new_shortcut in shortcuts:
-                    # Shortcut is here (either same action or not), don't replace in settings.
-                    replace = False
-                    if action_name != current_action:
-                        # the shortcut is saved to a different action
-
-                        # show warning symbols
-                        self._show_warning_icons([row, row1])
-
-                        # show warning message
-                        message = trans._(
-                            "The keybinding <b>{new_shortcut}</b>  is already assigned to <b>{action_description}</b>; change or clear that shortcut before assigning <b>{new_shortcut}</b> to this one.",
-                            new_shortcut=new_shortcut,
-                            action_description=action.description,
-                        )
-                        self._show_warning(new_shortcut, action, row, message)
-
-                        if len(current_shortcuts) > 0:
-                            # If there was a shortcut set originally, then format it and reset the text.
-                            format_shortcut = Shortcut(
-                                current_shortcuts[0]
-                            ).platform
-
-                            if format_shortcut != current_shortcuts[0]:
-                                # only skip the next round if there are special symbols
-                                self._skip = True
-
-                            current_item.setText(format_shortcut)
-
-                        else:
-                            # There wasn't a shortcut here.
-                            current_item.setText("")
-
-                        self._cleanup_warning_icons([row, row1])
-
-                        break
-
-                    else:
-                        # This shortcut was here.  Reformat and reset text.
-                        format_shortcut = Shortcut(new_shortcut).platform
-                        if format_shortcut != new_shortcut:
-                            # Only skip the next round if there are special symbols in shortcut.
-                            self._skip = True
-
-                        current_item.setText(format_shortcut)
+            replace = self._mark_conflicts(new_shortcut, row)
 
             if replace is True:
                 # This shortcut is not taken.
 
                 #  Unbind current action from shortcuts in action manager.
                 action_manager.unbind_shortcut(current_action)
-
+                shortcuts_list = list(current_shortcuts)
+                ind = col - self._shortcut_col
                 if new_shortcut != "":
-                    # Bind the new shortcut.
-                    try:
-                        action_manager.bind_shortcut(
-                            current_action, new_shortcut
-                        )
-                    except TypeError:
-                        # Shortcut is not valid.
-                        action_manager._shortcuts[current_action] = set()
-                        # need to rebind the old shortcut
-                        action_manager.unbind_shortcut(current_action)
-                        action_manager.bind_shortcut(
-                            current_action, current_shortcuts[0]
-                        )
+                    if ind < len(shortcuts_list):
+                        shortcuts_list[ind] = new_shortcut
+                    else:
+                        shortcuts_list.append(new_shortcut)
+                elif ind < len(shortcuts_list):
+                    shortcuts_list.pop(col - self._shortcut_col)
+                new_value_dict = {}
+                # Bind the new shortcut.
+                try:
+                    for short in shortcuts_list:
+                        action_manager.bind_shortcut(current_action, short)
+                except TypeError:
+                    self._show_bind_shortcut_error(
+                        current_action,
+                        current_shortcuts,
+                        row,
+                        new_shortcut,
+                    )
+                    return
 
-                        # Show warning message to let user know this shortcut is invalid.
-                        self._show_warning_icons([row])
+                # The new shortcut is valid and can be displayed in widget.
 
-                        message = trans._(
-                            "<b>{new_shortcut}</b> is not a valid keybinding.",
-                            new_shortcut=new_shortcut,
-                        )
-                        self._show_warning(new_shortcut, action, row, message)
+                # Keep track of what changed.
+                new_value_dict = {current_action: shortcuts_list}
 
-                        self._cleanup_warning_icons([row])
+                self._restore_shortcuts(row)
 
-                        format_shortcut = Shortcut(
-                            current_shortcuts[0]
-                        ).platform
-                        if format_shortcut != current_shortcuts[0]:
-                            # Skip the next round if there are special symbols.
-                            self._skip = True
-
-                        # Update text to formated shortcut.
-                        current_item.setText(format_shortcut)
-                        return
-
-                    # The new shortcut is valid and can be displayed in widget.
-
-                    # Keep track of what changed.
-                    new_value_dict = {current_action: [new_shortcut]}
-
-                    # Format new shortcut.
-                    format_shortcut = Shortcut(new_shortcut).platform
-                    if format_shortcut != new_shortcut:
-                        # Skip the next round because there are special symbols.
-                        self._skip = True
-
-                    # Update text to formated shortcut.
-                    current_item.setText(format_shortcut)
-
-                else:
-                    # There is not a new shortcut to bind.  Keep track of it.
-                    if action_manager._shortcuts[current_action] != "":
-                        new_value_dict = {current_action: [""]}
-
-                if new_value_dict:
-                    # Emit signal when new value set for shortcut.
-                    self.valueChanged.emit(new_value_dict)
+                # Emit signal when new value set for shortcut.
+                self.valueChanged.emit(new_value_dict)
 
     def _show_warning_icons(self, rows):
         """Creates and displays the warning icons.
@@ -492,9 +513,7 @@ class ShortcutEditor(QWidget):
 
         value = {}
 
-        for row, (action_name, action) in enumerate(
-            action_manager._actions.items()
-        ):
+        for action_name, action in action_manager._actions.items():
             shortcuts = action_manager._shortcuts.get(action_name, [])
             value[action_name] = list(shortcuts)
 
@@ -504,20 +523,22 @@ class ShortcutEditor(QWidget):
 class ShortcutDelegate(QItemDelegate):
     """Delegate that handles when user types in new shortcut."""
 
-    def createEditor(self, QWidget, QStyleOptionViewItem, QModelIndex):
-        self._editor = EditorWidget(QWidget)
+    def createEditor(self, widget, style_option, model_index):
+        self._editor = EditorWidget(widget)
         return self._editor
 
-    def setEditorData(self, widget, QModelIndex):
-        text = QModelIndex.model().data(QModelIndex, Qt.EditRole)
+    def setEditorData(self, widget, model_index):
+        text = model_index.model().data(model_index, Qt.ItemDataRole.EditRole)
         widget.setText(str(text) if text else "")
 
-    def updateEditorGeometry(self, QWidget, QStyleOptionViewItem, QModelIndex):
-        QWidget.setGeometry(QStyleOptionViewItem.rect)
+    def updateEditorGeometry(self, widget, style_option, model_index):
+        widget.setGeometry(style_option.rect)
 
-    def setModelData(self, widget, QAbstractItemModel, QModelIndex):
+    def setModelData(self, widget, abstract_item_model, model_index):
         text = widget.text()
-        QAbstractItemModel.setData(QModelIndex, text, Qt.EditRole)
+        abstract_item_model.setData(
+            model_index, text, Qt.ItemDataRole.EditRole
+        )
 
 
 class EditorWidget(QLineEdit):
@@ -528,10 +549,10 @@ class EditorWidget(QLineEdit):
 
     def event(self, event):
         """Qt method override."""
-        if event.type() == QEvent.ShortcutOverride:
+        if event.type() == QEvent.Type.ShortcutOverride:
             self.keyPressEvent(event)
             return True
-        elif event.type() in [QEvent.KeyPress, QEvent.Shortcut]:
+        elif event.type() in [QEvent.Type.KeyPress, QEvent.Type.Shortcut]:
             return True
         else:
             return super().event(event)
@@ -539,18 +560,35 @@ class EditorWidget(QLineEdit):
     def keyPressEvent(self, event):
         """Qt method override."""
         event_key = event.key()
-        if not event_key or event_key == Qt.Key_unknown:
+        if not event_key or event_key == Qt.Key.Key_unknown:
             return
 
-        if event_key in [
-            Qt.Key_Control,
-            Qt.Key_Shift,
-            Qt.Key_Alt,
-            Qt.Key_Meta,
-            Qt.Key_Return,
-            Qt.Key_Tab,
-            Qt.Key_CapsLock,
-        ]:
+        if (
+            event_key in {Qt.Key.Key_Delete, Qt.Key.Key_Backspace}
+            and self.text() != ''
+        ):
+            # Allow user to delete shortcut.
+            self.setText('')
+            return
+
+        key_map = {
+            Qt.Key.Key_Control: keys.CONTROL.name,
+            Qt.Key.Key_Shift: keys.SHIFT.name,
+            Qt.Key.Key_Alt: keys.ALT.name,
+            Qt.Key.Key_Meta: keys.META.name,
+            Qt.Key.Key_Delete: keys.DELETE.name,
+        }
+
+        if event_key in key_map:
+            self.setText(key_map[event_key])
+            return
+
+        if event_key in {
+            Qt.Key.Key_Return,
+            Qt.Key.Key_Tab,
+            Qt.Key.Key_CapsLock,
+            Qt.Key.Key_Enter,
+        }:
             # Do not allow user to set these keys as shortcut.
             return
 
@@ -562,16 +600,16 @@ class EditorWidget(QLineEdit):
         # Split the shortcut if it contains a symbol.
         parsed = re.split('[-(?=.+)]', event_keystr)
 
-        keys = []
+        keys_li = []
         # Format how the shortcut is written (ex. 'Ctrl+B' is changed to 'Control-B')
         for val in parsed:
             if val in KEY_SUBS.keys():
-                keys.append(KEY_SUBS[val])
+                keys_li.append(KEY_SUBS[val])
             else:
-                keys.append(val)
+                keys_li.append(val)
 
-        keys = '-'.join(keys)
-        self.setText(keys)
+        keys_li = '-'.join(keys_li)
+        self.setText(keys_li)
 
 
 class ShortcutTranslator(QKeySequenceEdit):
@@ -600,3 +638,13 @@ class ShortcutTranslator(QKeySequenceEdit):
     def event(self, event):
         """Qt Override"""
         return False
+
+
+@contextlib.contextmanager
+def lock_keybind_update(widget: ShortcutEditor):
+    prev = widget._skip
+    widget._skip = True
+    try:
+        yield
+    finally:
+        widget._skip = prev
