@@ -9,7 +9,10 @@ class NapariSplashScreen(QSplashScreen):
     def __init__(self, width=360):
         get_app()
         pm = QPixmap(NAPARI_ICON_PATH).scaled(
-            width, width, Qt.KeepAspectRatio, Qt.SmoothTransformation
+            width,
+            width,
+            Qt.AspectRatioMode.KeepAspectRatio,
+            Qt.TransformationMode.SmoothTransformation,
         )
         super().__init__(pm)
         self.show()
