@@ -1,8 +1,9 @@
+import contextlib
 import warnings
 from functools import reduce
 from itertools import count
 from operator import ior
-from typing import TYPE_CHECKING, List, Optional
+from typing import TYPE_CHECKING, List, Optional, Union
 from weakref import ReferenceType, ref
 
 from qtpy.QtCore import Qt
@@ -21,6 +22,8 @@ from ...utils.translations import trans
 from ..utils import combine_widgets, qt_signals_blocked
 
 if TYPE_CHECKING:
+    from magicgui.widgets import Widget
+
     from ..qt_viewer import QtViewer
 
 counter = count()
@@ -39,7 +42,7 @@ class QtViewerDockWidget(QDockWidget):
     ----------
     qt_viewer : QtViewer
         The QtViewer instance that this dock widget will belong to.
-    widget : QWidget
+    widget : QWidget or magicgui.widgets.Widget
         `widget` that will be added as QDockWidget's main widget.
     name : str
         Name of dock widget.
@@ -66,7 +69,7 @@ class QtViewerDockWidget(QDockWidget):
     def __init__(
         self,
         qt_viewer,
-        widget: QWidget,
+        widget: Union[QWidget, 'Widget'],
         *,
         name: str = '',
         area: str = 'right',
@@ -83,10 +86,10 @@ class QtViewerDockWidget(QDockWidget):
         self._close_btn = close_btn
 
         areas = {
-            'left': Qt.LeftDockWidgetArea,
-            'right': Qt.RightDockWidgetArea,
-            'top': Qt.TopDockWidgetArea,
-            'bottom': Qt.BottomDockWidgetArea,
+            'left': Qt.DockWidgetArea.LeftDockWidgetArea,
+            'right': Qt.DockWidgetArea.RightDockWidgetArea,
+            'top': Qt.DockWidgetArea.TopDockWidgetArea,
+            'bottom': Qt.DockWidgetArea.BottomDockWidgetArea,
         }
         if area not in areas:
             raise ValueError(
@@ -127,7 +130,7 @@ class QtViewerDockWidget(QDockWidget):
                 )
             allowed_areas = reduce(ior, [areas[a] for a in allowed_areas])
         else:
-            allowed_areas = Qt.AllDockWidgetAreas
+            allowed_areas = Qt.DockWidgetArea.AllDockWidgetAreas
         self.setAllowedAreas(allowed_areas)
         self.setMinimumHeight(50)
         self.setMinimumWidth(50)
@@ -166,7 +169,11 @@ class QtViewerDockWidget(QDockWidget):
 
     def destroyOnClose(self):
         """Destroys dock plugin dock widget when 'x' is clicked."""
-        self._ref_qt_viewer().viewer.window.remove_dock_widget(self)
+        from napari.viewer import Viewer
+
+        viewer = self._ref_qt_viewer().viewer
+        if isinstance(viewer, Viewer):
+            viewer.window.remove_dock_widget(self)
 
     def _maybe_add_vertical_stretch(self, widget):
         """Add vertical stretch to the bottom of a vertical layout only
@@ -223,7 +230,10 @@ class QtViewerDockWidget(QDockWidget):
         return self._ref_qt_viewer().keyPressEvent(event)
 
     def _set_title_orientation(self, area):
-        if area in (Qt.LeftDockWidgetArea, Qt.RightDockWidgetArea):
+        if area in (
+            Qt.DockWidgetArea.LeftDockWidgetArea,
+            Qt.DockWidgetArea.RightDockWidgetArea,
+        ):
             features = self._features
             if features & self.DockWidgetVerticalTitleBar:
                 features = features ^ self.DockWidgetVerticalTitleBar
@@ -237,29 +247,29 @@ class QtViewerDockWidget(QDockWidget):
             par = self.parent()
             if par and hasattr(par, 'dockWidgetArea'):
                 return par.dockWidgetArea(self) in (
-                    Qt.LeftDockWidgetArea,
-                    Qt.RightDockWidgetArea,
+                    Qt.DockWidgetArea.LeftDockWidgetArea,
+                    Qt.DockWidgetArea.RightDockWidgetArea,
                 )
         return self.size().height() > self.size().width()
 
     def _on_visibility_changed(self, visible):
-        try:
-            actions = [
-                action.text()
-                for action in self._ref_qt_viewer().viewer.window.plugins_menu.actions()
-            ]
-            idx = actions.index(self.name)
+        from napari.viewer import Viewer
+
+        with contextlib.suppress(AttributeError, ValueError):
+            viewer = self._ref_qt_viewer().viewer
+            if isinstance(viewer, Viewer):
+                actions = [
+                    action.text()
+                    for action in viewer.window.plugins_menu.actions()
+                ]
+                idx = actions.index(self.name)
+
+                viewer.window.plugins_menu.actions()[idx].setChecked(visible)
 
-            current_action = (
-                self._ref_qt_viewer().viewer.window.plugins_menu.actions()[idx]
-            )
-            current_action.setChecked(visible)
             self.setVisible(visible)
-
-        except (AttributeError, ValueError):
             # AttributeError: This error happens when the plugins menu is not yet built.
             # ValueError: This error is when the action is from the windows menu.
-            pass
+
         if not visible:
             return
         with qt_signals_blocked(self):
@@ -309,17 +319,17 @@ class QtCustomTitleBar(QLabel):
         self.hide_button = QPushButton(self)
         self.hide_button.setToolTip(trans._('hide this panel'))
         self.hide_button.setObjectName("QTitleBarHideButton")
-        self.hide_button.setCursor(Qt.ArrowCursor)
+        self.hide_button.setCursor(Qt.CursorShape.ArrowCursor)
         self.hide_button.clicked.connect(lambda: self.parent().close())
 
         self.float_button = QPushButton(self)
         self.float_button.setToolTip(trans._('float this panel'))
         self.float_button.setObjectName("QTitleBarFloatButton")
-        self.float_button.setCursor(Qt.ArrowCursor)
+        self.float_button.setCursor(Qt.CursorShape.ArrowCursor)
         self.float_button.clicked.connect(
             lambda: self.parent().setFloating(not self.parent().isFloating())
         )
-        self.title = QLabel(title, self)
+        self.title: QLabel = QLabel(title, self)
         self.title.setSizePolicy(
             QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Maximum)
         )
@@ -328,7 +338,7 @@ class QtCustomTitleBar(QLabel):
             self.close_button = QPushButton(self)
             self.close_button.setToolTip(trans._('close this panel'))
             self.close_button.setObjectName("QTitleBarCloseButton")
-            self.close_button.setCursor(Qt.ArrowCursor)
+            self.close_button.setCursor(Qt.CursorShape.ArrowCursor)
             self.close_button.clicked.connect(
                 lambda: self.parent().destroyOnClose()
             )
@@ -339,10 +349,16 @@ class QtCustomTitleBar(QLabel):
             layout.setContentsMargins(0, 8, 0, 8)
             line.setFixedWidth(1)
             if close_btn:
-                layout.addWidget(self.close_button, 0, Qt.AlignHCenter)
-            layout.addWidget(self.hide_button, 0, Qt.AlignHCenter)
-            layout.addWidget(self.float_button, 0, Qt.AlignHCenter)
-            layout.addWidget(line, 0, Qt.AlignHCenter)
+                layout.addWidget(
+                    self.close_button, 0, Qt.AlignmentFlag.AlignHCenter
+                )
+            layout.addWidget(
+                self.hide_button, 0, Qt.AlignmentFlag.AlignHCenter
+            )
+            layout.addWidget(
+                self.float_button, 0, Qt.AlignmentFlag.AlignHCenter
+            )
+            layout.addWidget(line, 0, Qt.AlignmentFlag.AlignHCenter)
             self.title.hide()
 
         else:
@@ -359,7 +375,7 @@ class QtCustomTitleBar(QLabel):
             layout.addWidget(self.title)
 
         self.setLayout(layout)
-        self.setCursor(Qt.OpenHandCursor)
+        self.setCursor(Qt.CursorShape.OpenHandCursor)
 
     def sizeHint(self):
         # this seems to be the correct way to set the height of the titlebar
