@@ -1,4 +1,5 @@
 """Test scale bar."""
+import numpy as np
 import pytest
 from pint import UndefinedUnitError
 
@@ -16,10 +17,10 @@ def test_vispy_text_visual(make_napari_viewer):
     assert qt_widget.scale_bar._quantity is None
 
     # check visible attribute
-    assert qt_widget.scale_bar.node.visible == viewer.scale_bar.visible
+    assert qt_widget.scale_bar.line_node.visible == viewer.scale_bar.visible
     viewer.scale_bar.visible = True
     assert (
-        qt_widget.scale_bar.node.visible
+        qt_widget.scale_bar.line_node.visible
         == qt_widget.scale_bar.text_node.visible
         == viewer.scale_bar.visible
         is True
@@ -38,14 +39,14 @@ def test_vispy_text_visual(make_napari_viewer):
 
     # check ticks attribute
     viewer.scale_bar.ticks = False
-    assert len(qt_widget.scale_bar.node._pos) == 2
+    assert len(qt_widget.scale_bar.line_node._pos) == 2
     viewer.scale_bar.ticks = True
-    assert len(qt_widget.scale_bar.node._pos) == 6
+    assert len(qt_widget.scale_bar.line_node._pos) == 6
 
     # check visible attribute
-    assert qt_widget.scale_bar.node.visible == viewer.scale_bar.visible
+    assert qt_widget.scale_bar.line_node.visible == viewer.scale_bar.visible
     viewer.scale_bar.visible = True
-    assert qt_widget.scale_bar.node.visible == viewer.scale_bar.visible
+    assert qt_widget.scale_bar.line_node.visible == viewer.scale_bar.visible
 
     # check position attribute
     for position in list(Position):
@@ -74,4 +75,39 @@ def test_vispy_text_visual(make_napari_viewer):
     viewer.scale_bar.visible = False
     viewer.scale_bar.unit = "pixel"
     assert qt_widget.scale_bar._quantity.units == "pixel"
+
+    # test box visible attribute
+    assert qt_widget.scale_bar.rect_node.visible == viewer.scale_bar.box
+    viewer.scale_bar.box = True
+    viewer.scale_bar.visible = True
+    assert (
+        qt_widget.scale_bar.line_node.visible
+        == qt_widget.scale_bar.text_node.visible
+        == qt_widget.scale_bar.rect_node.visible
+        == viewer.scale_bar.visible
+        == viewer.scale_bar.box
+        is True
+    )
+
+    # test color attributes
+    viewer.scale_bar.colored = True
+    for (rgba, color) in [
+        ((0.0, 1.0, 1.0, 1.0), "#00ffff"),  # check hex color
+        ((1.0, 1.0, 0.0, 1.0), (1.0, 1.0, 0.0)),  # check 3 tuple
+        ((1.0, 0.5, 0.0, 0.5), (1.0, 0.5, 0.0, 0.5)),  # check 4 tuple
+        ((1.0, 1.0, 1.0, 1.0), "white"),  # check text color
+    ]:
+        viewer.scale_bar.color = color
+        np.testing.assert_equal(viewer.scale_bar.color, np.asarray(rgba))
+
+    viewer.scale_bar.box = True
+    for (rgba, color) in [
+        ((0.0, 1.0, 1.0, 1.0), "#00ffff"),  # check hex color
+        ((1.0, 1.0, 0.0, 1.0), (1.0, 1.0, 0.0)),  # check 3 tuple
+        ((1.0, 0.5, 0.0, 0.5), (1.0, 0.5, 0.0, 0.5)),  # check 4 tuple
+        ((1.0, 1.0, 1.0, 1.0), "white"),  # check text color
+    ]:
+        viewer.scale_bar.box_color = color
+        np.testing.assert_equal(viewer.scale_bar.box_color, np.asarray(rgba))
+
     del qt_widget
