@@ -4,7 +4,7 @@ import numpy as np
 from vispy.scene.visuals import Text
 
 from napari.layers import Points, Shapes
-from napari.layers.utils._text_constants import TextMode
+from napari.layers.utils.string_encoding import ConstantStringEncoding
 
 
 def update_text(
@@ -29,9 +29,11 @@ def update_text(
     # This also acts as a minor optimization.
     if _has_visible_text(layer):
         text_values = layer._view_text
+        colors = layer._view_text_color
         coords, anchor_x, anchor_y = layer._view_text_coords
     else:
         text_values = np.array([''])
+        colors = np.zeros((4,), np.float32)
         coords = np.zeros((1, ndisplay))
         anchor_x = 'center'
         anchor_y = 'center'
@@ -56,7 +58,7 @@ def update_text(
 
     text_manager = layer.text
     node.rotation = text_manager.rotation
-    node.color = text_manager.color
+    node.color = colors
     node.font_size = text_manager.size
 
 
@@ -64,9 +66,10 @@ def _has_visible_text(layer: Union[Points, Shapes]) -> bool:
     text = layer.text
     if not text.visible:
         return False
-    if len(text.values) == 0:
-        return False
-    if text._mode == TextMode.NONE:
+    if (
+        isinstance(text.string, ConstantStringEncoding)
+        and text.string.constant == ''
+    ):
         return False
     if len(layer._indices_view) == 0:
         return False
