@@ -8,8 +8,6 @@ import numpy as np
 
 from ..layers import Layer
 from ..layers.image.image import _ImageBase
-from ..utils.context import create_context
-from ..utils.context._layerlist_context import LayerListContextKeys
 from ..utils.events.containers import SelectableEventedList
 from ..utils.naming import inc_name_count
 from ..utils.translations import trans
@@ -48,12 +46,12 @@ class LayerList(SelectableEventedList[Layer]):
         emitted when ``index`` is set from ``old_value`` to ``value``
     reordered : (value: self)
         emitted when the list is reordered (eg. moved/reversed).
-    selection.changed : (added: Set[_T], removed: Set[_T])
+    selection.events.changed : (added: Set[_T], removed: Set[_T])
         emitted when the set changes, includes item(s) that have been added
         and/or removed from the set.
-    selection.active : (value: _T)
+    selection.events.active : (value: _T)
         emitted when the current item has changed.
-    selection._current : (value: _T)
+    selection.events._current : (value: _T)
         emitted when the current item has changed. (Private event)
 
     """
@@ -64,6 +62,16 @@ class LayerList(SelectableEventedList[Layer]):
             basetype=Layer,
             lookup={str: lambda e: e.name},
         )
+
+        # TODO: figure out how to move this context creation bit.
+        # Ideally, the app should be aware of the layerlist, but not vice versa.
+        # This could probably be done by having the layerlist emit events that the app
+        # connects to, then the `_ctx` object would live on the app, (not here)
+        from .._app_model.context import create_context
+        from .._app_model.context._layerlist_context import (
+            LayerListContextKeys,
+        )
+
         self._ctx = create_context(self)
         if self._ctx is not None:  # happens during Viewer type creation
             self._ctx_keys = LayerListContextKeys(self._ctx)
@@ -126,39 +134,37 @@ class LayerList(SelectableEventedList[Layer]):
         layer = event.source
         layer.name = self._coerce_name(layer.name, layer)
 
+    def _ensure_unique(self, values, allow=()):
+        bad = set(self._list) - set(allow)
+        values = tuple(values) if isinstance(values, Iterable) else (values,)
+        for v in values:
+            if v in bad:
+                raise ValueError(
+                    trans._(
+                        "Layer '{v}' is already present in layer list",
+                        deferred=True,
+                        v=v,
+                    )
+                )
+        return values
+
+    def __setitem__(self, key, value):
+        old = self._list[key]
+        if isinstance(key, slice):
+            value = self._ensure_unique(value, old)
+        elif isinstance(key, int):
+            (value,) = self._ensure_unique((value,), (old,))
+        super().__setitem__(key, value)
+
     def insert(self, index: int, value: Layer):
         """Insert ``value`` before index."""
+        (value,) = self._ensure_unique((value,))
         new_layer = self._type_check(value)
         new_layer.name = self._coerce_name(new_layer.name)
         self._clean_cache()
         new_layer.events.set_data.connect(self._clean_cache)
         super().insert(index, new_layer)
 
-    def move_selected(self, index, insert):
-        """Reorder list by moving the item at index and inserting it
-        at the insert index. If additional items are selected these will
-        get inserted at the insert index too. This allows for rearranging
-        the list based on dragging and dropping a selection of items, where
-        index is the index of the primary item being dragged, and insert is
-        the index of the drop location, and the selection indicates if
-        multiple items are being dragged. If the moved layer is not selected
-        select it.
-
-        Parameters
-        ----------
-        index : int
-            Index of primary item to be moved
-        insert : int
-            Index that item(s) will be inserted at
-        """
-        if self[index] not in self.selection:
-            self.selection.select_only(self[index])
-            moving = [index]
-        else:
-            moving = [i for i, x in enumerate(self) if x in self.selection]
-        offset = insert >= index
-        self.move_multiple(moving, insert + offset)
-
     def toggle_selected_visibility(self):
         """Toggle visibility of selected layers"""
         for layer in self.selection:
@@ -259,21 +265,39 @@ class LayerList(SelectableEventedList[Layer]):
     def _get_step_size(self, layer_extent_list):
         if len(self) == 0:
             return np.ones(self.ndim)
-        else:
-            scales = [extent.step for extent in layer_extent_list]
-            min_scales = self._step_size_from_scales(scales)
-            return min_scales
 
-    @cached_property
-    def extent(self) -> Extent:
-        """Extent of layers in data and world coordinates."""
-        extent_list = [layer.extent for layer in self]
+        scales = [extent.step for extent in layer_extent_list]
+        return self._step_size_from_scales(scales)
+
+    def get_extent(self, layers: Iterable[Layer]) -> Extent:
+        """
+        Return extent for a given layer list.
+        This function is useful for calculating the extent of a subset of layers
+        when preparing and updating some supplementary layers.
+        For example see the cross Vectors layer in the `multiple_viewer_widget` example.
+
+        Parameters
+        ----------
+        layers : list of Layer
+            list of layers for which extent should be calculated
+
+        Returns
+        -------
+        extent : Extent
+             extent for selected layers
+        """
+        extent_list = [layer.extent for layer in layers]
         return Extent(
             data=None,
             world=self._get_extent_world(extent_list),
             step=self._get_step_size(extent_list),
         )
 
+    @cached_property
+    def extent(self) -> Extent:
+        """Extent of layers in data and world coordinates."""
+        return self.get_extent([x for x in self])
+
     @cached_property
     def _ranges(self) -> List[Tuple[float, float, float]]:
         """Get ranges for Dims.range in world coordinates.
@@ -434,7 +458,11 @@ class LayerList(SelectableEventedList[Layer]):
         """
         from ..plugins.io import save_layers
 
-        layers = list(self.selection) if selected else list(self)
+        layers = (
+            [x for x in self if x in self.selection]
+            if selected
+            else list(self)
+        )
 
         if selected:
             msg = trans._("No layers selected", deferred=True)
