@@ -11,6 +11,7 @@ from typing import (
     Any,
     Dict,
     List,
+    Mapping,
     Optional,
     Sequence,
     Set,
@@ -27,17 +28,22 @@ from ..errors import (
     NoAvailableReaderError,
     ReaderPluginError,
 )
-from ..layers import Image, Layer
+from ..layers import Image, Labels, Layer, Points, Shapes
 from ..layers._source import layer_source
 from ..layers.image._image_utils import guess_labels
+from ..layers.labels._labels_key_bindings import labels_fun_to_mode
+from ..layers.points._points_key_bindings import points_fun_to_mode
+from ..layers.shapes._shapes_key_bindings import shapes_fun_to_mode
 from ..layers.utils.stack_utils import split_channels
 from ..plugins.utils import get_potential_readers, get_preferred_reader
 from ..settings import get_settings
 from ..utils._register import create_func as create_add_method
+from ..utils.action_manager import action_manager
 from ..utils.colormaps import ensure_colormap
-from ..utils.context import Context, create_context
 from ..utils.events import Event, EventedModel, disconnect_events
+from ..utils.events.event import WarningEmitter
 from ..utils.key_bindings import KeymapProvider
+from ..utils.migrations import rename_argument
 from ..utils.misc import is_sequence
 from ..utils.mouse_bindings import MousemapProvider
 from ..utils.progress import progress
@@ -125,17 +131,24 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
     overlays: Overlays = Field(default_factory=Overlays, allow_mutation=False)
 
     help: str = ''
-    status: str = 'Ready'
+    status: Union[str, Dict] = 'Ready'
     tooltip: Tooltip = Field(default_factory=Tooltip, allow_mutation=False)
     theme: str = Field(default_factory=_current_theme)
     title: str = 'napari'
 
     # 2-tuple indicating height and width
     _canvas_size: Tuple[int, int] = (600, 800)
-    _ctx: Context
+    _ctx: Mapping
+    # To check if mouse is over canvas to avoid race conditions between
+    # different events systems
+    mouse_over_canvas: bool = False
 
     def __init__(self, title='napari', ndisplay=2, order=(), axis_labels=()):
         # max_depth=0 means don't look for parent contexts.
+        from .._app_model.context import create_context
+
+        # FIXME: just like the LayerList, this object should ideally be created
+        # elsewhere.  The app should know about the ViewerModel, but not vice versa.
         self._ctx = create_context(self, max_depth=0)
         # allow extra attributes during model initialization, useful for mixins
         self.__config__.extra = Extra.allow
@@ -167,7 +180,16 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         )
 
         # Add extra events - ideally these will be removed too!
-        self.events.add(layers_change=Event, reset_view=Event)
+        self.events.add(
+            layers_change=WarningEmitter(
+                trans._(
+                    "This event will be removed in 0.5.0. Please use viewer.layers.events instead",
+                    deferred=True,
+                ),
+                type="layers_change",
+            ),
+            reset_view=Event,
+        )
 
         # Connect events
         self.grid.events.connect(self.reset_view)
@@ -177,7 +199,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         self.dims.events.order.connect(self._update_layers)
         self.dims.events.order.connect(self.reset_view)
         self.dims.events.current_step.connect(self._update_layers)
-        self.cursor.events.position.connect(self._on_cursor_position_change)
+        self.cursor.events.position.connect(
+            self._update_status_bar_from_cursor
+        )
         self.layers.events.inserted.connect(self._on_add_layer)
         self.layers.events.removed.connect(self._on_remove_layer)
         self.layers.events.reordered.connect(self._on_grid_change)
@@ -324,6 +348,10 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             layer._slice_dims(
                 self.dims.point, self.dims.ndisplay, self.dims.order
             )
+        position = list(self.cursor.position)
+        for ind in self.dims.order[: -self.dims.ndisplay]:
+            position[ind] = self.dims.point[ind]
+        self.cursor.position = position
 
     def _on_active_layer(self, event):
         """Update viewer state for a new active layer."""
@@ -374,20 +402,14 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         """Set the viewer cursor_size with the `event.cursor_size` int."""
         self.cursor.size = event.cursor_size
 
-    def _on_cursor_position_change(self):
-        """Set the layer cursor position."""
-        with warnings.catch_warnings():
-            # Catch the deprecation warning on layer.position
-            warnings.filterwarnings(
-                'ignore',
-                message=str(
-                    trans._('layer.position is deprecated', deferred=True)
-                ),
-            )
-            for layer in self.layers:
-                layer.position = self.cursor.position
+    def _update_status_bar_from_cursor(self, event=None):
+        """Update the status bar based on the current cursor position.
 
+        This is generally used as a callback when cursor.position is updated.
+        """
         # Update status and help bar based on active layer
+        if not self.mouse_over_canvas:
+            return
         active = self.layers.selection.active
         if active is not None:
             self.status = active.get_status(
@@ -396,6 +418,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
                 dims_displayed=list(self.dims.displayed),
                 world=True,
             )
+
             self.help = active.help
             if self.tooltip.visible:
                 self.tooltip.text = active._get_tooltip_text(
@@ -404,6 +427,8 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
                     dims_displayed=list(self.dims.displayed),
                     world=True,
                 )
+        else:
+            self.status = 'Ready'
 
     def _on_grid_change(self):
         """Arrange the current layers is a 2D grid."""
@@ -464,6 +489,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         layer.events.shear.connect(self._on_layers_change)
         layer.events.affine.connect(self._on_layers_change)
         layer.events.name.connect(self.layers._update_name)
+        if hasattr(layer.events, "mode"):
+            layer.events.mode.connect(self._on_layer_mode_change)
+        self._layer_help_from_mode(layer)
 
         # Update dims and grid model
         self._on_layers_change()
@@ -477,6 +505,42 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             midpoint = [self.rounded_division(*_range) for _range in ranges]
             self.dims.set_point(range(len(ranges)), midpoint)
 
+    @staticmethod
+    def _layer_help_from_mode(layer: Layer):
+        """
+        Update layer help text base on layer mode.
+        """
+        layer_to_func_and_mode = {
+            Points: points_fun_to_mode,
+            Labels: labels_fun_to_mode,
+            Shapes: shapes_fun_to_mode,
+        }
+
+        help_li = []
+        shortcuts = get_settings().shortcuts.shortcuts
+
+        for fun, mode_ in layer_to_func_and_mode.get(layer.__class__, []):
+            if mode_ == layer.mode:
+                continue
+            action_name = f"napari:{fun.__name__}"
+            desc = action_manager._actions[action_name].description.lower()
+            if not shortcuts[action_name]:
+                continue
+            help_li.append(
+                trans._(
+                    "use <{shortcut}> for {desc}",
+                    shortcut=shortcuts[action_name][0],
+                    desc=desc,
+                )
+            )
+
+        layer.help = ", ".join(help_li)
+
+    def _on_layer_mode_change(self, event):
+        self._layer_help_from_mode(event.source)
+        if (active := self.layers.selection.active) is not None:
+            self.help = active.help
+
     def _on_remove_layer(self, event):
         """Disconnect old layer events.
 
@@ -519,6 +583,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         self.layers.append(layer)
         return layer
 
+    @rename_argument("interpolation", "interpolation2d", "0.6.0")
     def add_image(
         self,
         data=None,
@@ -528,7 +593,8 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         colormap=None,
         contrast_limits=None,
         gamma=1,
-        interpolation='nearest',
+        interpolation2d='nearest',
+        interpolation3d='linear',
         rendering='mip',
         depiction='volume',
         iso_threshold=0.5,
@@ -591,7 +657,13 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             If a list then must be same length as the axis that is being
             expanded as channels.
         interpolation : str or list
-            Interpolation mode used by vispy. Must be one of our supported
+            Deprecated, to be removed in 0.6.0
+        interpolation2d : str or list
+            Interpolation mode used by vispy in 2D. Must be one of our supported
+            modes. If a list then must be same length as the axis that is being
+            expanded as channels.
+        interpolation3d : str or list
+            Interpolation mode used by vispy in 3D. Must be one of our supported
             modes. If a list then must be same length as the axis that is being
             expanded as channels.
         rendering : str or list
@@ -695,7 +767,8 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             'colormap': colormap,
             'contrast_limits': contrast_limits,
             'gamma': gamma,
-            'interpolation': interpolation,
+            'interpolation2d': interpolation2d,
+            'interpolation3d': interpolation3d,
             'rendering': rendering,
             'depiction': depiction,
             'iso_threshold': iso_threshold,
@@ -731,7 +804,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         if channel_axis is None:
             kwargs['colormap'] = kwargs['colormap'] or 'gray'
             kwargs['blending'] = kwargs['blending'] or 'translucent_no_depth'
-            # Helpful message if someone tries to add mulit-channel kwargs,
+            # Helpful message if someone tries to add multi-channel kwargs,
             # but forget the channel_axis arg
             for k, v in kwargs.items():
                 if k not in iterable_kwargs and is_sequence(v):
@@ -778,7 +851,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         reader_plugin : str, optional
             reader plugin to pass to viewer.open (only used if the sample data
             is a string).  by default None.
-        ``**kwargs``
+        **kwargs
             additional kwargs will be passed to the sample data loader provided
             by `plugin`.  Use of ``**kwargs`` may raise an error if the kwargs do
             not match the sample data loader.
@@ -804,6 +877,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
                 data = plugin_manager._sample_data[plugin][sample]['data']
             except KeyError:
                 available += list(plugin_manager.available_samples())
+        # npe2 uri sample data, extract the path so we can use viewer.open
+        elif hasattr(data.__self__, 'uri'):
+            data = data.__self__.uri
 
         if data is None:
             msg = trans._(
@@ -853,8 +929,8 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         self,
         path: PathOrPaths,
         *,
-        stack: bool = False,
-        plugin: Optional[str] = 'builtins',
+        stack: Union[bool, List[List[str]]] = False,
+        plugin: Optional[str] = 'napari',
         layer_type: Optional[str] = None,
         **kwargs,
     ) -> List[Layer]:
@@ -868,12 +944,14 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         ----------
         path : str or list of str
             A filepath, directory, or URL (or a list of any) to open.
-        stack : bool, optional
-            If a list of strings is passed and ``stack`` is ``True``, then the
+        stack : bool or list[list[str]], optional
+            If a list of strings is passed as ``path`` and ``stack`` is ``True``, then the
             entire list will be passed to plugins.  It is then up to individual
             plugins to know how to handle a list of paths.  If ``stack`` is
             ``False``, then the ``path`` list is broken up and passed to plugin
             readers one by one.  by default False.
+            If the stack option is a list of lists containing individual paths,
+            the inner lists are passedto the reader and will be stacked.
         plugin : str, optional
             Name of a plugin to use, by default builtins.  If provided, will
             force ``path`` to be read with the specified ``plugin``.
@@ -886,7 +964,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             additional) ``kwargs`` provided to this function.  This *may*
             result in exceptions if the data returned from the path is not
             compatible with the layer_type.
-        ``**kwargs``
+        **kwargs
             All other keyword arguments will be passed on to the respective
             ``add_layer`` method.
 
@@ -895,27 +973,28 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         layers : list
             A list of any layers that were added to the viewer.
         """
+        if plugin == 'builtins':
+            warnings.warn(
+                trans._(
+                    'The "builtins" plugin name is deprecated and will not work in a future version. Please use "napari" instead.',
+                    deferred=True,
+                ),
+            )
+            plugin = 'napari'
 
-        paths: List[str | Path] = (
+        paths: List[str | Path | List[str | Path]] = (
             [os.fspath(path)]
             if isinstance(path, (Path, str))
             else [os.fspath(p) for p in path]
         )
 
-        if stack:
-            if plugin:
-                layers = self._add_layers_with_plugins(
-                    paths,
-                    kwargs=kwargs,
-                    plugin=plugin,
-                    layer_type=layer_type,
-                    stack=stack,
-                )
-            else:
-                layers = self._open_or_raise_error(
-                    paths, kwargs, layer_type, stack
-                )
-            return layers
+        # If stack is a bool and True, add an additional layer of nesting.
+        if isinstance(stack, bool) and stack:
+            paths = [paths]
+
+        # If stack is a list and True, extend the paths with the inner lists.
+        elif isinstance(stack, list) and stack:
+            paths.extend(stack)
 
         added: List[Layer] = []  # for layers that get added
         with progress(
@@ -926,20 +1005,24 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             else None,  # indeterminate bar for 1 file
         ) as pbr:
             for _path in pbr:
+                # If _path is a list, set stack to True
+                _stack = True if isinstance(_path, list) else False
+                # If _path is not a list already, make it a list.
+                _path = [_path] if not isinstance(_path, list) else _path
                 if plugin:
                     added.extend(
                         self._add_layers_with_plugins(
-                            [_path],
+                            _path,
                             kwargs=kwargs,
                             plugin=plugin,
                             layer_type=layer_type,
-                            stack=stack,
+                            stack=_stack,
                         )
                     )
                 # no plugin choice was made
                 else:
                     layers = self._open_or_raise_error(
-                        [_path], kwargs, layer_type, stack
+                        _path, kwargs, layer_type, _stack
                     )
                     added.extend(layers)
 
@@ -950,7 +1033,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         paths: List[Union[Path, str]],
         kwargs: Dict[str, Any] = {},
         layer_type: Optional[str] = None,
-        stack: bool = False,
+        stack: Union[bool, List[List[str]]] = False,
     ):
         """Open paths if plugin choice is unambiguous, raising any errors.
 
@@ -978,8 +1061,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             keyword arguments to pass to layer adding method, by default {}
         layer_type : Optional[str], optional
             layer type for paths, by default None
-        stack : bool, optional
-            True if files should be opened as a stack, by default False
+        stack : bool or list[list[str]], optional
+            True if files should be opened as a stack, by default False.
+            Can also be a list containing lists of files to stack.
 
         Returns
         -------
@@ -1000,13 +1084,11 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             when multiple readers are available to read the path
         """
         paths = [os.fspath(path) for path in paths]  # PathObjects -> str
-
         added = []
         plugin = None
         _path = paths[0]
         # we want to display the paths nicely so make a help string here
         path_message = f"[{_path}], ...]" if len(paths) > 1 else _path
-
         readers = get_potential_readers(_path)
         if not readers:
             raise NoAvailableReaderError(
@@ -1023,7 +1105,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             warnings.warn(
                 RuntimeWarning(
                     trans._(
-                        f"Can't find {plugin} plugin associated with {path_message} files. ",
+                        "Can't find {plugin} plugin associated with {path_message} files. ",
                         plugin=plugin,
                         path_message=path_message,
                     )
