@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import itertools
+import os.path
 import warnings
 from abc import ABC, abstractmethod
 from collections import defaultdict, namedtuple
@@ -10,9 +11,14 @@ from typing import List, Optional, Tuple, Union
 
 import magicgui as mgui
 import numpy as np
+from npe2 import plugin_manager as pm
 
 from ...utils._dask_utils import configure_dask
-from ...utils._magicgui import add_layer_to_viewer, get_layers
+from ...utils._magicgui import (
+    add_layer_to_viewer,
+    add_layers_to_viewer,
+    get_layers,
+)
 from ...utils.events import EmitterGroup, Event
 from ...utils.events.event import WarningEmitter
 from ...utils.geometry import (
@@ -22,10 +28,9 @@ from ...utils.geometry import (
 from ...utils.key_bindings import KeymapProvider
 from ...utils.mouse_bindings import MousemapProvider
 from ...utils.naming import magic_name
-from ...utils.status_messages import generate_layer_status
+from ...utils.status_messages import generate_layer_coords_status
 from ...utils.transforms import Affine, CompositeAffine, TransformChain
 from ...utils.translations import trans
-from .._source import current_source
 from ..utils.interactivity_utils import drag_data_to_projected_distance
 from ..utils.layer_utils import (
     coerce_affine,
@@ -96,7 +101,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
     blending : str
         One of a list of preset blending modes that determines how RGB and
         alpha values of the layer visual get mixed. Allowed values are
-        {'opaque', 'translucent', 'translucent_no_depth', and 'additive'}.
+        {'opaque', 'translucent', 'translucent_no_depth', 'additive', and 'minimum'}.
     visible : bool
         Whether the layer visual is currently being displayed.
     multiscale : bool
@@ -121,17 +126,25 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         * ``Blending.TRANSLUCENT``
           Allows for multiple layers to be blended with different opacity and
           corresponds to ``depth_test=True``, ``cull_face=False``,
-          ``blend=True``, ``blend_func=('src_alpha', 'one_minus_src_alpha')``.
+          ``blend=True``, ``blend_func=('src_alpha', 'one_minus_src_alpha')``,
+          and ``blend_equation=('func_add')``.
         * ``Blending.TRANSLUCENT_NO_DEPTH``
           Allows for multiple layers to be blended with different opacity, but
           no depth testing is performed. Corresponds to ``depth_test=False``,
           ``cull_face=False``, ``blend=True``,
-          ``blend_func=('src_alpha', 'one_minus_src_alpha')``.
+          ``blend_func=('src_alpha', 'one_minus_src_alpha')``, and
+          ``blend_equation=('func_add')``.
         * ``Blending.ADDITIVE``
           Allows for multiple layers to be blended together with different
           colors and opacity. Useful for creating overlays. It corresponds to
           ``depth_test=False``, ``cull_face=False``, ``blend=True``,
-          ``blend_func=('src_alpha', 'one')``.
+          ``blend_func=('src_alpha', 'one')``, and ``blend_equation=('func_add')``.
+        * ``Blending.MINIMUM``
+            Allows for multiple layers to be blended together such that
+            the minimum of each RGB component and alpha are selected.
+            Useful for creating overlays with inverted colormaps. It
+            corresponds to ``depth_test=False``, ``cull_face=False``, ``blend=True``,
+            ``blend_equation=('min')``.
     scale : tuple of float
         Scale factors for the layer.
     translate : tuple of float
@@ -160,15 +173,11 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         Currently, this only applies to dask arrays.
     z_index : int
         Depth of the layer visual relative to other visuals in the scenecanvas.
-    coordinates : tuple of float
-        Cursor position in data coordinates.
     corner_pixels : array
         Coordinates of the top-left and bottom-right canvas pixels in the data
         coordinates of each layer. For multiscale data the coordinates are in
         the space of the currently viewed data level, not the highest resolution
         level.
-    position : tuple
-        Cursor position in world coordinates.
     ndim : int
         Dimensionality of the layer.
     thumbnail : (N, M, 4) array
@@ -226,6 +235,19 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         if name is None and data is not None:
             name = magic_name(data)
 
+        if scale is not None and not np.all(scale):
+            raise ValueError(
+                trans._(
+                    "Layer {name} is invalid because it has scale values of 0. The layer's scale is currently {scale}",
+                    deferred=True,
+                    name=repr(name),
+                    scale=repr(scale),
+                )
+            )
+
+        # Needs to be imported here to avoid circular import in _source
+        from .._source import current_source
+
         self._source = current_source()
         self.dask_optimized_slicing = configure_dask(data, cache)
         self._metadata = dict(metadata or {})
@@ -282,7 +304,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             ]
         )
 
-        self._position = (0,) * ndim
         self._dims_point = [0] * ndim
         self.corner_pixels = np.zeros((2, ndim), dtype=int)
         self._editable = True
@@ -463,12 +484,24 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         Blending.TRANSLUCENT
             Allows for multiple layers to be blended with different opacity
             and corresponds to depth_test=True, cull_face=False,
-            blend=True, blend_func=('src_alpha', 'one_minus_src_alpha').
+            blend=True, blend_func=('src_alpha', 'one_minus_src_alpha'),
+            and blend_equation=('func_add').
+        Blending.TRANSLUCENT_NO_DEPTH
+          Allows for multiple layers to be blended with different opacity, but
+          no depth testing is performed. Corresponds to ``depth_test=False``,
+          cull_face=False, blend=True, blend_func=('src_alpha', 'one_minus_src_alpha'),
+          and blend_equation=('func_add').
         Blending.ADDITIVE
             Allows for multiple layers to be blended together with
             different colors and opacity. Useful for creating overlays. It
             corresponds to depth_test=False, cull_face=False, blend=True,
-            blend_func=('src_alpha', 'one').
+            blend_func=('src_alpha', 'one'), and blend_equation=('func_add').
+        Blending.MINIMUM
+            Allows for multiple layers to be blended together such that
+            the minimum of each RGB component and alpha are selected.
+            Useful for creating overlays with inverted colormaps. It
+            corresponds to depth_test=False, cull_face=False, blend=True,
+            blend_equation=('min').
         """
         return str(self._blending)
 
@@ -657,7 +690,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             self._dims_order = list(
                 reorder_after_dim_reduction(self._dims_order[-ndim:])
             )
-            self._position = self._position[-ndim:]
         elif old_ndim < ndim:
             new_axes = range(ndim - old_ndim)
             self._transforms = self._transforms.expand_dims(new_axes)
@@ -665,7 +697,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
             self._dims_order = list(range(ndim - old_ndim)) + [
                 o + ndim - old_ndim for o in self._dims_order
             ]
-            self._position = (0,) * (ndim - old_ndim) + self._position
 
         self._ndim = ndim
         if 'extent' in self.__dict__:
@@ -1011,7 +1042,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
     def get_value(
         self,
-        position,
+        position: Tuple[float],
         *,
         view_direction: Optional[np.ndarray] = None,
         dims_displayed: Optional[List[int]] = None,
@@ -1023,7 +1054,7 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         Parameters
         ----------
-        position : tuple
+        position : tuple of float
             Position in either data or world coordinates.
         view_direction : Optional[np.ndarray]
             A unit vector giving the direction of the ray in nD world coordinates.
@@ -1558,28 +1589,27 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
         )
 
         # find the maximal data-axis-aligned bounding box containing all four
-        # canvas corners
+        # canvas corners and round them to ints
         data_bbox = np.stack(
             [np.min(data_corners, axis=0), np.max(data_corners, axis=0)]
         )
-        # round and clip the bounding box values
         data_bbox_int = np.stack(
             [np.floor(data_bbox[0]), np.ceil(data_bbox[1])]
         ).astype(int)
-        displayed_extent = self.extent.data[:, displayed_axes]
-        data_bbox_clipped = np.clip(
-            data_bbox_int, displayed_extent[0], displayed_extent[1]
-        )
 
         if self._ndisplay == 2 and self.multiscale:
             level, scaled_corners = compute_multiscale_level_and_corners(
-                data_bbox_clipped,
+                data_bbox_int,
                 shape_threshold,
                 self.downsample_factors[:, displayed_axes],
             )
-            corners = np.zeros((2, self.ndim))
-            corners[:, displayed_axes] = scaled_corners
-            corners = corners.astype(int)
+            corners = np.zeros((2, self.ndim), dtype=int)
+            # The corner_pixels attribute stores corners in the data
+            # space of the selected level. Using the level's data
+            # shape only works for images, but that's the only case we
+            # handle now and downsample_factors is also only on image layers.
+            max_coords = np.take(self.data[level].shape, displayed_axes)
+            corners[:, displayed_axes] = np.clip(scaled_corners, 0, max_coords)
             display_shape = tuple(
                 corners[1, displayed_axes] - corners[0, displayed_axes]
             )
@@ -1593,24 +1623,77 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                 self.refresh()
 
         else:
+            # The stored corner_pixels attribute must contain valid indices.
+            displayed_extent = self.extent.data[:, displayed_axes]
+            data_bbox_clipped = np.clip(
+                data_bbox_int, displayed_extent[0], displayed_extent[1]
+            )
             corners = np.zeros((2, self.ndim), dtype=int)
             corners[:, displayed_axes] = data_bbox_clipped
             self.corner_pixels = corners
 
+    def _get_source_info(self):
+        components = {}
+        if self.source.reader_plugin:
+            components['layer_base'] = os.path.basename(self.source.path or '')
+            components['source_type'] = 'plugin'
+            try:
+                components['plugin'] = pm.get_manifest(
+                    self.source.reader_plugin
+                ).display_name
+            except KeyError:
+                components['plugin'] = self.source.reader_plugin
+            return components
+
+        elif self.source.sample:
+            components['layer_base'] = self.name
+            components['source_type'] = 'sample'
+            try:
+                components['plugin'] = pm.get_manifest(
+                    self.source.sample[0]
+                ).display_name
+            except KeyError:
+                components['plugin'] = self.source.sample[0]
+            return components
+
+        elif self.source.widget:
+            components['layer_base'] = self.name
+            components['source_type'] = 'widget'
+            components['plugin'] = self.source.widget._function.__name__
+            return components
+
+        else:
+            components['layer_base'] = self.name
+            components['source_type'] = ''
+            components['plugin'] = ''
+            return components
+
+    def get_source_str(self):
+
+        source_info = self._get_source_info()
+
+        return (
+            source_info['layer_base']
+            + ', '
+            + source_info['source_type']
+            + ' : '
+            + source_info['plugin']
+        )
+
     def get_status(
         self,
-        position: np.ndarray,
+        position: Optional[Tuple[float, ...]] = None,
         *,
         view_direction: Optional[np.ndarray] = None,
         dims_displayed: Optional[List[int]] = None,
         world=False,
     ):
         """
-        Status message of the data at a coordinate position.
+        Status message information of the data at a coordinate position.
 
         Parameters
         ----------
-        position : tuple
+        position : tuple of float
             Position in either data or world coordinates.
         view_direction : Optional[np.ndarray]
             A unit vector giving the direction of the ray in nD world coordinates.
@@ -1624,16 +1707,24 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
 
         Returns
         -------
-        msg : string
-            String containing a message that can be used as a status update.
+        source_info : dict
+            Dictionary containing a information that can be used as a status update.
         """
-        value = self.get_value(
-            position,
-            view_direction=view_direction,
-            dims_displayed=dims_displayed,
-            world=world,
+        if position is not None:
+            value = self.get_value(
+                position,
+                view_direction=view_direction,
+                dims_displayed=dims_displayed,
+                world=world,
+            )
+        else:
+            value = None
+
+        source_info = self._get_source_info()
+        source_info['coordinates'] = generate_layer_coords_status(
+            position, value
         )
-        return generate_layer_status(self.name, position, value)
+        return source_info
 
     def _get_tooltip_text(
         self,
@@ -1782,3 +1873,6 @@ class Layer(KeymapProvider, MousemapProvider, ABC):
                     layer_type=layer_type,
                 )
             ) from exc
+
+
+mgui.register_type(type_=List[Layer], return_callback=add_layers_to_viewer)
