@@ -47,8 +47,19 @@ def select(layer, event):
             layer.selected_data = set()
     layer._set_highlight()
 
+    # Set _drag_start value here to prevent an offset when mouse_move happens
+    # https://github.com/napari/napari/pull/4999
+    layer._set_drag_start(
+        layer.selected_data,
+        layer.world_to_data(event.position),
+        center_by_data=not modify_selection,
+    )
     yield
 
+    # Undo the toggle selected in case of a mouse move with modifiers
+    if modify_selection and value is not None and event.type == 'mouse_move':
+        layer.selected_data = _toggle_selected(layer.selected_data, value)
+
     is_moving = False
     # on move
     while event.type == 'mouse_move':
@@ -62,8 +73,6 @@ def select(layer, event):
             # while dragging, update the drag box
             coord = [coordinates[i] for i in layer._dims_displayed]
             layer._is_selecting = True
-            if layer._drag_start is None:
-                layer._drag_start = coord
             layer._drag_box = np.array([layer._drag_start, coord])
 
             # update the drag up and normal vectors on the layer
@@ -89,6 +98,7 @@ def select(layer, event):
         )
 
     # reset the selection box data and highlights
+    layer._drag_box = None
     layer._drag_normal = None
     layer._drag_up = None
     layer._set_highlight(force=True)
