@@ -44,6 +44,10 @@ def select(layer, event):
 
     # we don't update the thumbnail unless a shape has been moved
     update_thumbnail = False
+
+    # Set _drag_start value here to prevent an offset when mouse_move happens
+    # https://github.com/napari/napari/pull/4999
+    _set_drag_start(layer, layer.world_to_data(event.position))
     yield
 
     # on move
@@ -359,6 +363,14 @@ def _drag_selection_box(layer, coordinates):
     layer._set_highlight()
 
 
+def _set_drag_start(layer, coordinates):
+    coord = [coordinates[i] for i in layer._dims_displayed]
+    if layer._drag_start is None and len(layer.selected_data) > 0:
+        center = layer._selected_box[Box.CENTER]
+        layer._drag_start = coord - center
+    return coord
+
+
 def _move(layer, coordinates):
     """Moves object at given mouse position and set of indices.
 
@@ -378,14 +390,11 @@ def _move(layer, coordinates):
     if layer._mode in (
         [Mode.SELECT, Mode.ADD_RECTANGLE, Mode.ADD_ELLIPSE, Mode.ADD_LINE]
     ):
-        coord = [coordinates[i] for i in layer._dims_displayed]
+        coord = _set_drag_start(layer, coordinates)
         layer._moving_coordinates = coordinates
         layer._is_moving = True
         if vertex is None:
             # Check where dragging box from to move whole object
-            if layer._drag_start is None:
-                center = layer._selected_box[Box.CENTER]
-                layer._drag_start = coord - center
             center = layer._selected_box[Box.CENTER]
             shift = coord - center - layer._drag_start
             for index in layer.selected_data:
