@@ -1,3 +1,4 @@
+import warnings
 from typing import TypeVar
 
 from ...translations import trans
@@ -77,10 +78,53 @@ class SelectableEventedList(Selectable[_T], EventedList[_T]):
         for i in list(self.selection):
             idx = self.index(i)
             self.remove(i)
-        new = max(0, (idx - 1))
-        if len(self) > new:
+        if isinstance(idx, int):
+            new = max(0, (idx - 1))
+            do_add = len(self) > new
+        else:
+            *root, _idx = idx
+            new = tuple(root) + (_idx - 1,) if _idx >= 1 else tuple(root)
+            do_add = len(self) > new[0]
+        if do_add:
             self.selection.add(self[new])
 
+    def move_selected(self, index: int, insert: int):
+        """Reorder list by moving the item at index and inserting it
+        at the insert index. If additional items are selected these will
+        get inserted at the insert index too. This allows for rearranging
+        the list based on dragging and dropping a selection of items, where
+        index is the index of the primary item being dragged, and insert is
+        the index of the drop location, and the selection indicates if
+        multiple items are being dragged. If the moved layer is not selected
+        select it.
+
+        This method is deprecated. Please use layers.move_multiple
+        with layers.selection instead.
+
+        Parameters
+        ----------
+        index : int
+            Index of primary item to be moved
+        insert : int
+            Index that item(s) will be inserted at
+        """
+        # this is just here for now to support the old layerlist API
+        warnings.warn(
+            trans._(
+                'move_selected is deprecated since 0.4.16. Please use layers.move_multiple with layers.selection instead.',
+                deferred=True,
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
+        if self[index] not in self.selection:
+            self.selection.select_only(self[index])
+            moving = [index]
+        else:
+            moving = [i for i, x in enumerate(self) if x in self.selection]
+        offset = insert >= index
+        self.move_multiple(moving, insert + offset)
+
     def select_next(self, step=1, shift=False):
         """Selects next item from list."""
         if self.selection:
