@@ -186,8 +186,7 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
         # https://pydantic-docs.helpmanual.io/usage/models/#private-model-attributes
         underscore_attrs_are_private = True
         # whether to validate field defaults (default: False)
-        # see https://github.com/napari/napari/pull/4138 before changing.
-        validate_all = False
+        validate_all = True
         # https://pydantic-docs.helpmanual.io/usage/exporting_models/#modeljson
         # NOTE: json_encoders are also added EventedMetaclass.__new__ if the
         # field declares a _json_encode method.
@@ -198,13 +197,21 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
 
         self._events.source = self
         # add event emitters for each field which is mutable
-        event_names = [
+        field_events = [
             name
             for name, field in self.__fields__.items()
             if field.field_info.allow_mutation
         ]
-        event_names.extend(self.__property_setters__)
-        self._events.add(**dict.fromkeys(event_names))
+
+        self._events.add(
+            **dict.fromkeys(field_events + list(self.__property_setters__))
+        )
+
+        # while seemingly redundant, this next line is very important to maintain
+        # correct sources; see https://github.com/napari/napari/pull/4138
+        # we solve it by re-setting the source after initial validation, which allows
+        # us to use `validate_all = True`
+        self._reset_event_source()
 
     def _super_setattr_(self, name: str, value: Any) -> None:
         # pydantic will raise a ValueError if extra fields are not allowed
@@ -242,6 +249,21 @@ class EventedModel(BaseModel, metaclass=EventedMetaclass):
     def events(self) -> EmitterGroup:
         return self._events
 
+    def _reset_event_source(self):
+        """
+        set the event sources of self and all the children to the correct values
+        """
+        # events are all messed up due to objects being probably
+        # recreated arbitrarily during validation
+        self.events.source = self
+        for name in self.__fields__:
+            child = getattr(self, name)
+            if isinstance(child, EventedModel):
+                # TODO: this isinstance check should be EventedMutables in the future
+                child._reset_event_source()
+            elif name in self.events.emitters:
+                getattr(self.events, name).source = self
+
     @property
     def _defaults(self):
         return get_defaults(self)
