@@ -76,10 +76,6 @@ def sys_info(as_html=False):
     as_html : bool
         if True, info will be returned as HTML, suitable for a QTextEdit widget
     """
-    from npe2 import PluginManager as Npe2PluginManager
-
-    from napari.plugins import plugin_manager
-
     sys_version = sys.version.replace('\n', ' ')
     text = (
         f"<b>napari</b>: {napari.__version__}<br>"
@@ -115,6 +111,11 @@ def sys_info(as_html=False):
         ('scipy', 'SciPy'),
         ('dask', 'Dask'),
         ('vispy', 'VisPy'),
+        ('magicgui', 'magicgui'),
+        ('superqt', 'superqt'),
+        ('in_n_out', 'in-n-out'),
+        ('app_model', 'app-model'),
+        ('npe2', 'npe2'),
     )
 
     loaded = {}
@@ -153,32 +154,6 @@ def sys_info(as_html=False):
     except Exception as e:
         text += f"  - failed to load screen information {e}"
 
-    plugin_manager.discover()
-    plugin_strings = {}
-    for meta in plugin_manager.list_plugin_metadata():
-        plugin_name = meta.get('plugin_name')
-        if plugin_name == 'builtins':
-            continue
-        version = meta.get('version')
-        version_string = f": {version}" if version else ""
-        plugin_strings[plugin_name] = f"  - {plugin_name}{version_string}"
-
-    npe2_plugin_manager = Npe2PluginManager.instance()
-    for manifest in npe2_plugin_manager.iter_manifests():
-        plugin_name = manifest.name
-        if plugin_name in ("napari", "builtins"):
-            continue
-        version = manifest.package_version
-        version_string = f": {version}" if version else ""
-        plugin_strings[plugin_name] = f"  - {plugin_name}{version_string}"
-
-    text += '<br><b>Plugins</b>:'
-    text += (
-        ("<br>" + "<br>".join(sorted(plugin_strings.values())))
-        if plugin_strings
-        else '  None'
-    )
-
     if not as_html:
         text = (
             text.replace("<br>", "\n").replace("<b>", "").replace("</b>", "")
