@@ -1,3 +1,4 @@
+import sys
 import typing
 from typing import TYPE_CHECKING, Optional
 from weakref import WeakSet
@@ -19,21 +20,24 @@ class Viewer(ViewerModel):
     Parameters
     ----------
     title : string, optional
-        The title of the viewer window. by default 'napari'.
+        The title of the viewer window. By default 'napari'.
     ndisplay : {2, 3}, optional
-        Number of displayed dimensions. by default 2.
+        Number of displayed dimensions. By default 2.
     order : tuple of int, optional
         Order in which dimensions are displayed where the last two or last
         three dimensions correspond to row x column or plane x row x column if
-        ndisplay is 2 or 3. by default None
+        ndisplay is 2 or 3. By default None
     axis_labels : list of str, optional
-        Dimension names. by default they are labeled with sequential numbers
+        Dimension names. By default they are labeled with sequential numbers
     show : bool, optional
-        Whether to show the viewer after instantiation. by default True.
+        Whether to show the viewer after instantiation. By default True.
     """
 
     _window: 'Window' = None  # type: ignore
-    _instances: typing.ClassVar[WeakSet] = WeakSet()
+    if sys.version_info < (3, 9):
+        _instances: typing.ClassVar[WeakSet] = WeakSet()
+    else:
+        _instances: typing.ClassVar[WeakSet['Viewer']] = WeakSet()
 
     def __init__(
         self,
@@ -50,10 +54,16 @@ class Viewer(ViewerModel):
             order=order,
             axis_labels=axis_labels,
         )
+        # we delay initialization of plugin system to the first instantiation
+        # of a viewer... rather than just on import of plugins module
+        from .plugins import _initialize_plugins
+
         # having this import here makes all of Qt imported lazily, upon
         # instantiating the first Viewer.
         from .window import Window
 
+        _initialize_plugins()
+
         self._window = Window(self, show=show)
         self._instances.add(self)
 
@@ -80,13 +90,27 @@ class Viewer(ViewerModel):
         else:
             self.window._qt_viewer.console.push(variables)
 
-    def screenshot(self, path=None, *, canvas_only=True, flash: bool = True):
+    def screenshot(
+        self,
+        path=None,
+        *,
+        size=None,
+        scale=None,
+        canvas_only=True,
+        flash: bool = True,
+    ):
         """Take currently displayed screen and convert to an image array.
 
         Parameters
         ----------
         path : str
             Filename for saving screenshot image.
+        size : tuple (int, int)
+            Size (resolution) of the screenshot. By default, the currently displayed size.
+            Only used if `canvas_only` is True.
+        scale : float
+            Scale factor used to increase resolution of canvas for the screenshot. By default, the currently displayed resolution.
+            Only used if `canvas_only` is True.
         canvas_only : bool
             If True, screenshot shows only the image display canvas, and
             if False include the napari viewer frame in the screenshot,
@@ -103,7 +127,11 @@ class Viewer(ViewerModel):
             upper-left corner of the rendered region.
         """
         return self.window.screenshot(
-            path=path, flash=flash, canvas_only=canvas_only
+            path=path,
+            size=size,
+            scale=scale,
+            flash=flash,
+            canvas_only=canvas_only,
         )
 
     def show(self, *, block=False):
@@ -140,7 +168,7 @@ class Viewer(ViewerModel):
 
         Returns
         -------
-        int :
+        int
             number of viewer closed.
 
         """
